#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Módulo para seguimiento de progreso de proyectos.

Este módulo permite analizar un proyecto para determinar el progreso 
en diversas dimensiones: completitud, calidad, testing, branches, etc.
Provee datos para alimentar el dashboard de progreso.
"""

import os
import re
import json
from typing import Dict, List, Any, Optional, Tuple
from datetime import datetime
import subprocess

from src.analyzers.project_scanner import ProjectScanner
from src.analyzers.dependency_graph import DependencyGraph
from src.analyzers.completeness_verifier import CompletenessVerifier
from src.analyzers.file_analyzer import FileAnalyzer
from src.utils.logger import get_logger
from src.utils.config import ConfigManager
from src.utils.subscription_manager import get_subscription_manager

# Configuración del logger
logger = get_logger()


class ProjectProgressTracker:
    """
    Analizador que rastrea el progreso de un proyecto en múltiples dimensiones.
    
    Esta clase permite obtener métricas de avance del proyecto, estado de
    branches y otras informaciones para visualizar en un dashboard.
    """

    def __init__(self, project_path: str, config: Optional[ConfigManager] = None):
        """
        Inicializar el rastreador de progreso.
        
        Args:
            project_path: Ruta al directorio del proyecto
            config: Configuración opcional
        """
        self.project_path = os.path.abspath(project_path)
        self.config = config or ConfigManager()
        self.subscription = get_subscription_manager()
        
        # Inicializar analizadores
        self.scanner = ProjectScanner(project_path)
        self.dependency_graph = DependencyGraph()  # No necesita project_path como argumento
        self.completeness_verifier = CompletenessVerifier(config)  # Solo requiere config
        
        # FileAnalyzer espera un valor float, no un objeto ConfigManager
        max_file_size = 5.0  # Valor predeterminado en MB
        if config and hasattr(config, 'get'):
            # Intentar obtener del config si tiene método get
            try:
                max_file_size = float(config.get('analyzer', {}).get('max_file_size_mb', 5.0))
            except (ValueError, AttributeError, TypeError):
                pass
                
        self.file_analyzer = FileAnalyzer(max_file_size)
        
        # Guardar project_path para cuando los analizadores lo necesiten
        self.project_path = project_path
        
        # Caché para resultados
        self._cache = {}
        
        # Verificar acceso premium para funcionalidades avanzadas
        self.premium_access = self.subscription.is_premium_feature_available('project_dashboard')
    
    def get_project_overview(self) -> Dict[str, Any]:
        """
        Obtener información general del proyecto.
        
        Returns:
            Diccionario con información general del proyecto
        """
        # Si ya está en caché y tiene menos de 5 minutos, devolverlo
        if 'overview' in self._cache:
            timestamp, data = self._cache['overview']
            if (datetime.now() - timestamp).seconds < 300:  # 5 minutos
                return data
        
        # Escanear el proyecto
        project_data = self.scanner.scan_project(self.project_path)
        self.scanner.files = project_data.get('files', [])  # Asegurar que files esté disponible
        
        # Información básica
        overview = {
            "name": os.path.basename(self.project_path),
            "path": self.project_path,
            "last_updated": self._get_last_updated(),
            "files": {
                "total": len(self.scanner.files),
                "by_extension": self._count_files_by_extension()
            },
            "code_metrics": self._get_code_metrics(),
            "structure": self._get_structure_info(),
            "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }
        
        # Guardar en caché
        self._cache['overview'] = (datetime.now(), overview)
        
        return overview
    
    def get_progress_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas de progreso del proyecto.
        
        Returns:
            Diccionario con métricas de progreso
        """
        # Si ya está en caché y tiene menos de 5 minutos, devolverlo
        if 'progress' in self._cache:
            timestamp, data = self._cache['progress']
            if (datetime.now() - timestamp).seconds < 300:  # 5 minutos
                return data
        
        # Obtener métricas
        progress = {
            "completeness": self._get_completeness_metrics(),
            "code_quality": self._get_code_quality_metrics(),
            "testing": self._get_testing_metrics(),
            "version_control": self._get_git_metrics()
        }
        
        # Solo para usuarios premium: métricas avanzadas
        if self.premium_access:
            progress["advanced"] = self._get_advanced_metrics()
        
        # Guardar en caché
        self._cache['progress'] = (datetime.now(), progress)
        
        return progress
    
    def get_branch_status(self) -> Dict[str, Any]:
        """
        Obtener estado de las branches del proyecto.
        
        Returns:
            Diccionario con información de branches
        """
        # Verificar si el directorio es un repositorio git
        git_dir = os.path.join(self.project_path, '.git')
        if not os.path.isdir(git_dir):
            return {"error": "No es un repositorio Git"}
        
        try:
            # Obtener todas las branches
            output = subprocess.check_output(
                ['git', 'branch', '-a', '--sort=-committerdate'],
                cwd=self.project_path,
                stderr=subprocess.STDOUT,
                universal_newlines=True
            )
            
            branches = []
            for line in output.splitlines():
                line = line.strip()
                is_current = line.startswith('*')
                if is_current:
                    branch_name = line[2:].strip()
                else:
                    branch_name = line.strip()
                
                # Si es una branch remota, extraer el nombre
                if branch_name.startswith('remotes/'):
                    parts = branch_name.split('/')
                    if len(parts) > 2:
                        branch_name = '/'.join(parts[2:])
                
                # Filtrar HEAD y otras entradas especiales
                if branch_name.endswith('HEAD') or not branch_name:
                    continue
                
                # Obtener fecha del último commit
                try:
                    date_output = subprocess.check_output(
                        ['git', 'log', '-1', '--format=%ad', '--date=iso', branch_name],
                        cwd=self.project_path,
                        stderr=subprocess.DEVNULL,
                        universal_newlines=True
                    ).strip()
                    
                    # Obtener mensaje del último commit
                    msg_output = subprocess.check_output(
                        ['git', 'log', '-1', '--format=%s', branch_name],
                        cwd=self.project_path,
                        stderr=subprocess.DEVNULL,
                        universal_newlines=True
                    ).strip()
                    
                except subprocess.CalledProcessError:
                    date_output = "N/A"
                    msg_output = "N/A"
                
                branches.append({
                    "name": branch_name,
                    "current": is_current,
                    "last_commit_date": date_output,
                    "last_commit_msg": msg_output
                })
            
            # Organizar branches por categorías típicas de desarrollo
            branch_categories = {
                "feature": [b for b in branches if "feature" in b["name"]],
                "bugfix": [b for b in branches if "bugfix" in b["name"] or "fix" in b["name"]],
                "release": [b for b in branches if "release" in b["name"]],
                "main": [b for b in branches if b["name"] in ("main", "master")],
                "develop": [b for b in branches if b["name"] in ("develop", "dev")],
                "other": [b for b in branches if not any(
                    x in b["name"] for x in ("feature", "bugfix", "fix", "release")) 
                    and b["name"] not in ("main", "master", "develop", "dev")]
            }
            
            return {
                "branches": branches,
                "categories": branch_categories,
                "count": len(branches)
            }
            
        except subprocess.CalledProcessError as e:
            return {
                "error": f"Error al obtener información de branches: {str(e)}",
                "branches": []
            }
    
    def get_feature_progress(self) -> Dict[str, Any]:
        """
        Analizar el progreso de las características del proyecto usando grupos funcionales.
        
        Returns:
            Dict con información sobre el progreso de características funcionales
        """
        # Esta función requiere acceso premium
        if not self.premium_access:
            return {
                "error": "Esta función requiere suscripción premium",
                "features": []
            }
        
        try:
            # Intentar usar análisis en caché para evitar análisis duplicados
            from src.analyzers.analysis_cache import get_analysis_cache
            cache = get_analysis_cache()
            
            # Usar cache primero si está disponible
            cached_result = cache.get(self.project_path, 'dependencies')
            
            if cached_result:
                logger.info(f"✅ Usando análisis de dependencias en caché para grupos funcionales")
                analysis_result = cached_result
            else:
                # Sin caché, ejecutar análisis
                from src.analyzers.dependency_graph import DependencyGraph
                
                dep_analyzer = DependencyGraph()
                analysis_result = dep_analyzer.build_dependency_graph(self.project_path)
                
                # Guardar en caché para futuras llamadas
                cache.set(self.project_path, 'dependencies', analysis_result)
            
            # Extraer grupos funcionales del análisis
            functional_groups = analysis_result.get('functionality_groups', [])
            
            features = {}
            
            # Convertir grupos funcionales en formato de características
            for group in functional_groups:
                group_name = group.get('name', 'Unknown Group')
                group_files = group.get('files', [])
                group_type = group.get('type', 'unknown')
                
                # Limpiar nombre del grupo para evitar duplicados de emojis
                clean_name = group_name.replace('📁 ', '').replace('🔧 ', '').replace('🔗 ', '').replace('🎨 ', '').replace('🧪 ', '')
                
                # Asegurar que tenemos archivos y un nombre válido
                if group_files and clean_name.strip() and len(clean_name.strip()) > 2:
                    # Extraer paths de archivos si están en formato de diccionario
                    file_paths = []
                    for file_item in group_files:
                        if isinstance(file_item, dict):
                            path = file_item.get('path', file_item.get('file', ''))
                            if path:
                                file_paths.append(path)
                        elif isinstance(file_item, str):
                            file_paths.append(file_item)
                    
                    # Solo añadir el grupo si tiene archivos válidos
                    if file_paths:
                        features[clean_name] = {
                            "type": group_type,
                            "files": len(file_paths),
                            "file_list": file_paths,  # Guardar lista real de archivos
                            "description": group.get('description', f"Grupo funcional: {clean_name}"),
                            "importance": group.get('total_importance', 0),
                            "size": group.get('size', len(file_paths)),
                            "completion_estimate": self._estimate_group_completion(file_paths)
                        }
            
            # Si no hay grupos funcionales válidos, crear grupos básicos basados en funcionalidades detectadas
            if not features:
                features = self._create_basic_functional_groups()
                
        except Exception as e:
            logger.error(f"Error al analizar grupos funcionales: {e}")
            # Fallback a grupos básicos
            features = self._create_basic_functional_groups()
        
        # Detectar características basadas en archivos de especificación
        feature_specs = self._find_feature_specs()
        for feature_name, spec in feature_specs.items():
            if feature_name not in features:
                features[feature_name] = {
                    "has_spec": True,
                    "spec_file": spec.get("file", ""),
                    "completion_estimate": self._check_spec_implementation(spec)
                }
        
        return {
            "features": features,
            "count": len(features)
        }
    
    def get_recommendations(self) -> List[Dict[str, Any]]:
        """
        Genera recomendaciones proactivas para mejorar el proyecto.
        
        Returns:
            Lista de recomendaciones
        """
        # Esta función requiere acceso premium
        if not self.premium_access:
            return [{"message": "Las recomendaciones proactivas requieren suscripción premium"}]
        
        recommendations = []
        
        # Obtener métricas para basar recomendaciones
        progress = self.get_progress_metrics()
        code_quality = progress.get("code_quality", {})
        testing = progress.get("testing", {})
        
        # Revisión de cobertura de tests
        test_coverage = testing.get("coverage", 0)
        if test_coverage < 50:
            recommendations.append({
                "type": "testing",
                "priority": "high" if test_coverage < 30 else "medium",
                "message": f"Baja cobertura de tests ({test_coverage}%). Considera añadir más pruebas.",
                "action": "Ejecuta 'project-prompt test [módulo]' para generar pruebas unitarias."
            })
        
        # Revisión de documentación
        doc_percentage = code_quality.get("documentation_percentage", 0)
        if doc_percentage < 40:
            recommendations.append({
                "type": "documentation",
                "priority": "medium",
                "message": f"Documentación insuficiente ({doc_percentage}%). Mejora la documentación del código.",
                "action": "Añade docstrings y comentarios a clases y funciones principales."
            })
        
        # Revisión de branches
        branches = self.get_branch_status()
        old_branches = [b for b in branches.get("branches", []) if self._is_branch_old(b.get("last_commit_date", ""))]
        if old_branches:
            branch_names = ", ".join([b["name"] for b in old_branches[:3]])
            more = f" y {len(old_branches)-3} más" if len(old_branches) > 3 else ""
            recommendations.append({
                "type": "version_control",
                "priority": "medium",
                "message": f"Detectadas {len(old_branches)} branches sin actividad reciente: {branch_names}{more}",
                "action": "Considera fusionar o eliminar branches obsoletas."
            })
        
        # Revisión de complejidad
        complex_files = code_quality.get("complex_files", [])
        if complex_files:
            file_names = ", ".join([os.path.basename(f["file"]) for f in complex_files[:2]])
            more = f" y {len(complex_files)-2} más" if len(complex_files) > 2 else ""
            recommendations.append({
                "type": "code_quality",
                "priority": "high",
                "message": f"Detectados {len(complex_files)} archivos con alta complejidad: {file_names}{more}",
                "action": "Refactoriza estos archivos para reducir su complejidad."
            })
        
        return recommendations
    
    def _get_last_updated(self) -> str:
        """
        Obtener fecha de última actualización del proyecto.
        
        Returns:
            Fecha de última modificación en formato ISO
        """
        try:
            # Intentar obtener la última fecha de commit
            output = subprocess.check_output(
                ['git', 'log', '-1', '--format=%ad', '--date=iso'],
                cwd=self.project_path,
                stderr=subprocess.DEVNULL,
                universal_newlines=True
            ).strip()
            return output
        except (subprocess.CalledProcessError, OSError):
            # Si falla, usar la fecha de modificación del directorio
            return datetime.fromtimestamp(
                os.path.getmtime(self.project_path)
            ).strftime("%Y-%m-%d %H:%M:%S")
    
    def _count_files_by_extension(self) -> Dict[str, int]:
        """
        Contar archivos por extensión.
        
        Returns:
            Diccionario con conteo de archivos por extensión
        """
        extensions = {}
        for file in self.scanner.files:
            # Si es un diccionario, usar la extensión directamente
            if isinstance(file, dict) and 'extension' in file:
                ext = file['extension'].lower() or "sin extensión"
            else:
                # Si es una ruta, extraer la extensión
                file_path = self._get_file_path(file)
                ext = os.path.splitext(file_path)[1].lower() or "sin extensión"
                
            extensions[ext] = extensions.get(ext, 0) + 1
        return extensions
    
    def _get_code_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas generales de código.
        
        Returns:
            Diccionario con métricas de código
        """
        code_files = [f for f in self.scanner.files if self._is_code_file(f)]
        
        total_lines = 0
        code_lines = 0
        comment_lines = 0
        
        for file in code_files:
            try:
                file_path = self._get_file_path(file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    lines = f.readlines()
                
                total_lines += len(lines)
                
                # Contar líneas de código y comentarios de forma simplificada
                for line in lines:
                    line = line.strip()
                    if not line:
                        continue
                    if line.startswith('#') or line.startswith('//') or line.startswith('/*') or line.startswith('*'):
                        comment_lines += 1
                    else:
                        code_lines += 1
            except Exception as e:
                logger.warning(f"Error al analizar archivo {file}: {str(e)}")
        
        return {
            "total_lines": total_lines,
            "code_lines": code_lines,
            "comment_lines": comment_lines,
            "files": len(code_files)
        }
    
    def _get_structure_info(self) -> Dict[str, Any]:
        """
        Obtener información sobre la estructura del proyecto.
        
        Returns:
            Diccionario con información de estructura
        """
        # Analizar estructura de directorios
        dirs = {}
        for root, directories, files in os.walk(self.project_path):
            # Excluir directorios ocultos y virtuales
            if any(part.startswith('.') for part in root.split(os.sep)) or \
               any(d in root for d in ['node_modules', 'venv', '__pycache__']):
                continue
            
            rel_path = os.path.relpath(root, self.project_path)
            if rel_path == '.':
                rel_path = ''
            
            files = [f for f in files if not f.startswith('.')]
            dirs[rel_path] = len(files)
        
        # Calcular profundidad máxima
        max_depth = 0
        for path in dirs:
            depth = len(path.split(os.sep)) if path else 0
            max_depth = max(max_depth, depth)
        
        return {
            "directories": len(dirs),
            "max_depth": max_depth,
            "dir_structure": dirs
        }
    
    def _get_completeness_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas de completitud del proyecto.
        
        Returns:
            Diccionario con métricas de completitud
        """
        # Crear resultado simulado ya que no tenemos el método verify_project_completeness
        # En una implementación real, se usaría el método correspondiente del verificador
        result = {
            "score": 75,  # Porcentaje estimado de completitud
            "missing_components": ["documentación extendida", "tests de integración"],
            "implemented_components": ["core", "UI", "análisis", "tests unitarios"],
            "suggestions": ["Completar los tests de integración", "Mejorar la documentación"]
        }
        
        return {
            "score": result.get("score", 0),
            "missing_components": result.get("missing_components", []),
            "implemented_components": result.get("implemented_components", []),
            "suggestions": result.get("suggestions", [])
        }
    
    def _get_code_quality_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas de calidad de código.
        
        Returns:
            Diccionario con métricas de calidad
        """
        code_files = [f for f in self.scanner.files if self._is_code_file(f)]
        
        # Métricas por defecto
        metrics = {
            "documentation_percentage": self._calculate_documentation_percentage(code_files),
            "complex_files": self._find_complex_files(code_files),
            "duplication_estimate": self._estimate_code_duplication(code_files)
        }
        
        return metrics
    
    def _get_testing_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas relacionadas con pruebas.
        
        Returns:
            Diccionario con métricas de pruebas
        """
        # Buscar archivos de prueba
        test_files = self._find_test_files()
        code_files = [f for f in self.scanner.files if self._is_code_file(f) and f not in test_files]
        
        # Métricas básicas
        metrics = {
            "test_files": len(test_files),
            "code_files": len(code_files),
            "ratio": len(test_files) / max(len(code_files), 1),
            "coverage": self._estimate_test_coverage(test_files, code_files)
        }
        
        return metrics
    
    def _get_git_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas relacionadas con el control de versiones.
        
        Returns:
            Diccionario con métricas de Git
        """
        try:
            # Verificar si es un repositorio git
            if not os.path.isdir(os.path.join(self.project_path, '.git')):
                return {"is_git_repo": False}
            
            # Obtener número de commits
            commits_count = subprocess.check_output(
                ['git', 'rev-list', '--count', 'HEAD'],
                cwd=self.project_path,
                stderr=subprocess.DEVNULL,
                universal_newlines=True
            ).strip()
            
            # Obtener contribuyentes
            contributors = subprocess.check_output(
                ['git', 'shortlog', '-sn', '--no-merges'],
                cwd=self.project_path,
                stderr=subprocess.DEVNULL,
                universal_newlines=True
            ).strip().split('\n')
            
            contributors_list = []
            for contributor in contributors:
                if not contributor.strip():
                    continue
                parts = contributor.strip().split('\t', 1)
                if len(parts) == 2:
                    count, name = parts
                    contributors_list.append({
                        "name": name.strip(),
                        "commits": int(count.strip())
                    })
            
            # Obtener actividad reciente
            activity = subprocess.check_output(
                ['git', 'log', '--format=%ad', '--date=short', '-n', '14'],
                cwd=self.project_path,
                stderr=subprocess.DEVNULL,
                universal_newlines=True
            ).strip().split('\n')
            
            # Contar commits por día
            activity_count = {}
            for date in activity:
                if date:
                    activity_count[date] = activity_count.get(date, 0) + 1
            
            # Ordenar por fecha
            activity_sorted = [{"date": k, "commits": v} for k, v in activity_count.items()]
            activity_sorted.sort(key=lambda x: x["date"])
            
            return {
                "is_git_repo": True,
                "commits": int(commits_count),
                "contributors": contributors_list,
                "recent_activity": activity_sorted
            }
            
        except (subprocess.CalledProcessError, OSError) as e:
            logger.warning(f"Error al obtener métricas de Git: {str(e)}")
            return {"is_git_repo": True, "error": str(e)}
    
    def _get_advanced_metrics(self) -> Dict[str, Any]:
        """
        Obtener métricas avanzadas (solo para usuarios premium).
        
        Returns:
            Diccionario con métricas avanzadas
        """
        # Usar cache para evitar análisis duplicado de dependencias
        if not hasattr(self, '_dependency_graph_cache'):
            logger.info("Construyendo grafo de dependencias (primera vez)...")
            self._dependency_graph_cache = self.dependency_graph.build_dependency_graph(self.project_path)
        else:
            logger.info("Usando grafo de dependencias cacheado...")
        
        graph_data = self._dependency_graph_cache
        
        # Identificar módulos centrales (alto número de dependientes)
        central_modules = []
        central_files = graph_data.get('central_files', [])
        for central_file in central_files[:5]:  # Limitar a 5
            central_modules.append({
                "file": central_file.get('file', ''),
                "dependents": central_file.get('in_degree', 0)  # in_degree represents dependents
            })
        
        # Identificar arquitectura del proyecto
        architecture = self._detect_architecture_pattern()
        
        return {
            "central_modules": central_modules,
            "architecture_pattern": architecture,
            "modularity_score": self._calculate_modularity_score(),
            "risk_areas": self._identify_risk_areas()
        }
    
    def _is_code_file(self, file) -> bool:
        """
        Determinar si un archivo es código.
        
        Args:
            file: Ruta al archivo (str) o diccionario de información de archivo
            
        Returns:
            True si es un archivo de código, False en caso contrario
        """
        code_extensions = {
            '.py', '.js', '.ts', '.jsx', '.tsx', '.java', '.c', '.cpp', '.cs',
            '.go', '.rb', '.php', '.swift', '.kt', '.rs', '.scala', '.sh'
        }
        
        # Si es un diccionario, obtener la extensión del campo correspondiente
        if isinstance(file, dict) and 'extension' in file:
            return file['extension'].lower() in code_extensions
        
        # Si es una ruta (str), obtener la extensión del path
        elif isinstance(file, str):
            return os.path.splitext(file)[1].lower() in code_extensions
            
        return False
    
    def _find_test_files(self) -> List[str]:
        """
        Encontrar archivos de prueba en el proyecto.
        
        Returns:
            Lista de rutas a archivos de prueba
        """
        test_files = []
        
        for file in self.scanner.files:
            file_path = self._get_file_path(file)
            file_lower = file_path.lower()
            if 'test' in os.path.basename(file_lower) or \
               '/test' in file_lower.replace('\\', '/') or \
               '/tests/' in file_lower.replace('\\', '/'):
                test_files.append(file)
        
        return test_files
    
    def _calculate_documentation_percentage(self, files: List[str]) -> float:
        """
        Calcular porcentaje de documentación en los archivos.
        
        Args:
            files: Lista de rutas a archivos
            
        Returns:
            Porcentaje de documentación (0-100)
        """
        total_functions = 0
        documented_functions = 0
        
        for file in files:
            try:
                file_path = self._get_file_path(file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                # Detectar lenguaje
                ext = os.path.splitext(file_path)[1].lower()
                
                # Para Python
                if ext == '.py':
                    # Contar definiciones de funciones y clases
                    func_matches = re.findall(r'def\s+\w+\s*\(', content)
                    class_matches = re.findall(r'class\s+\w+', content)
                    total_functions += len(func_matches) + len(class_matches)
                    
                    # Contar docstrings
                    docstring_matches = re.findall(r'"""[\s\S]*?"""', content)
                    documented_functions += len(docstring_matches)
                
                # Para JavaScript/TypeScript
                elif ext in ('.js', '.ts', '.jsx', '.tsx'):
                    # Contar definiciones de funciones
                    func_matches = re.findall(r'function\s+\w+\s*\(|const\s+\w+\s*=\s*\(|let\s+\w+\s*=\s*\(|var\s+\w+\s*=\s*\(', content)
                    class_matches = re.findall(r'class\s+\w+', content)
                    total_functions += len(func_matches) + len(class_matches)
                    
                    # Contar comentarios JSDoc
                    jsdoc_matches = re.findall(r'/\*\*[\s\S]*?\*/', content)
                    documented_functions += len(jsdoc_matches)
                
                # Para otros lenguajes, simplificar
                else:
                    # Contar líneas que parecen definiciones de funciones
                    func_matches = re.findall(r'\w+\s*\(.*?\)\s*\{', content)
                    total_functions += len(func_matches)
                    
                    # Contar bloques de comentarios
                    comment_matches = re.findall(r'/\*[\s\S]*?\*/|//.*?$', content, re.MULTILINE)
                    documented_functions += min(len(comment_matches), len(func_matches))
                
            except Exception as e:
                logger.warning(f"Error al analizar documentación de {file}: {str(e)}")
        
        # Calcular porcentaje
        return (documented_functions / max(total_functions, 1)) * 100
    
    def _find_complex_files(self, files: List[str]) -> List[Dict[str, Any]]:
        """
        Encontrar archivos con alta complejidad.
        
        Args:
            files: Lista de rutas a archivos
            
        Returns:
            Lista de archivos complejos con métricas
        """
        complex_files = []
        
        for file in files:
            try:
                file_path = self._get_file_path(file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                # Métricas simples de complejidad
                metrics = {
                    "file": file_path,
                    "lines": content.count('\n') + 1,
                    "functions": self._count_functions(content),
                    "nested_depth": self._estimate_nesting_depth(content)
                }
                
                # Determinar si es complejo
                is_complex = (
                    metrics["lines"] > 500 or
                    metrics["functions"] > 20 or
                    metrics["nested_depth"] > 5
                )
                
                if is_complex:
                    complex_files.append(metrics)
                
            except Exception as e:
                logger.warning(f"Error al analizar complejidad de {file}: {str(e)}")
        
        # Ordenar por complejidad
        return sorted(complex_files, 
                      key=lambda x: x["lines"] + x["functions"] * 10 + x["nested_depth"] * 30,
                      reverse=True)
    
    def _count_functions(self, content: str) -> int:
        """
        Contar funciones en el contenido de un archivo.
        
        Args:
            content: Contenido del archivo
            
        Returns:
            Número de funciones
        """
        # Buscar patrones comunes de definición de funciones
        patterns = [
            r'def\s+\w+\s*\(',  # Python
            r'function\s+\w+\s*\(',  # JavaScript
            r'const\s+\w+\s*=\s*\(',  # Arrow functions
            r'let\s+\w+\s*=\s*\(',
            r'var\s+\w+\s*=\s*\(',
            r'class\s+\w+',  # Clases
            r'\w+\s*\(.*?\)\s*\{',  # Estilo C/Java
        ]
        
        count = 0
        for pattern in patterns:
            count += len(re.findall(pattern, content))
        
        return count
    
    def _estimate_nesting_depth(self, content: str) -> int:
        """
        Estimar profundidad de anidación máxima.
        
        Args:
            content: Contenido del archivo
            
        Returns:
            Profundidad máxima estimada
        """
        lines = content.split('\n')
        max_depth = 0
        current_depth = 0
        
        for line in lines:
            stripped = line.strip()
            
            # Incrementar profundidad al abrir bloques
            if stripped.endswith('{') or stripped.endswith(':'):
                current_depth += 1
            
            # Decrementar profundidad al cerrar bloques
            elif stripped == '}' or stripped.startswith('}'):
                current_depth = max(0, current_depth - 1)
            
            max_depth = max(max_depth, current_depth)
        
        return max_depth
    
    def _estimate_code_duplication(self, files: List[str]) -> Dict[str, Any]:
        """
        Estimar duplicación de código en el proyecto.
        
        Args:
            files: Lista de rutas a archivos
            
        Returns:
            Estimación de duplicación
        """
        # Enfoque simplificado: detectar funciones con nombres similares
        function_names = {}
        
        for file in files:
            try:
                file_path = self._get_file_path(file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                # Extraer nombres de funciones
                # Python
                for match in re.finditer(r'def\s+(\w+)\s*\(', content):
                    name = match.group(1)
                    function_names[name] = function_names.get(name, 0) + 1
                
                # JavaScript/TypeScript
                for match in re.finditer(r'function\s+(\w+)\s*\(', content):
                    name = match.group(1)
                    function_names[name] = function_names.get(name, 0) + 1
                
            except Exception as e:
                logger.warning(f"Error al analizar duplicación en {file}: {str(e)}")
        
        # Contar funciones duplicadas
        duplicated = {name: count for name, count in function_names.items() if count > 1}
        
        # Calcular porcentaje aproximado
        total_functions = sum(function_names.values())
        duplicated_functions = sum(duplicated.values()) - len(duplicated)
        percentage = (duplicated_functions / max(total_functions, 1)) * 100
        
        return {
            "percentage": percentage,
            "duplicated_functions": len(duplicated),
            "examples": list(duplicated.keys())[:5]
        }
    
    def _estimate_test_coverage(self, test_files: List[str], code_files: List[str]) -> float:
        """
        Estimar cobertura de pruebas.
        
        Args:
            test_files: Lista de archivos de prueba
            code_files: Lista de archivos de código
            
        Returns:
            Porcentaje estimado de cobertura
        """
        # Extraer nombres de funciones de los archivos de código
        code_functions = set()
        
        for file in code_files:
            try:
                file_path = self._get_file_path(file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                # Python
                for match in re.finditer(r'def\s+(\w+)\s*\(', content):
                    code_functions.add(match.group(1))
                
                # JavaScript/TypeScript
                for match in re.finditer(r'function\s+(\w+)\s*\(', content):
                    code_functions.add(match.group(1))
                
                # Clases
                for match in re.finditer(r'class\s+(\w+)', content):
                    code_functions.add(match.group(1))
                
            except Exception:
                pass
        
        # Buscar esas funciones en los archivos de prueba
        tested_functions = set()
        
        for file in test_files:
            try:
                file_path = self._get_file_path(file)
                with open(file_path, 'r', encoding='utf-8') as f:
                    content = f.read()
                
                for func in code_functions:
                    if func in content:
                        tested_functions.add(func)
                
            except Exception:
                pass
        
        # Calcular porcentaje
        if not code_functions:
            return 0
        
        return (len(tested_functions) / len(code_functions)) * 100
    
    def _get_files_in_dir(self, directory: str) -> List[str]:
        """
        Obtener rutas a todos los archivos en un directorio y subdirectorios.
        
        Args:
            directory: Directorio a analizar
            
        Returns:
            Lista de rutas a archivos
        """
        files = []
        
        for root, _, filenames in os.walk(directory):
            for filename in filenames:
                # Excluir archivos ocultos y binarios
                if not filename.startswith('.') and self._is_text_file(filename):
                    files.append(os.path.join(root, filename))
        
        return files
    
    def _is_text_file(self, filename: str) -> bool:
        """
        Determinar si un archivo es de texto basado en su extensión.
        
        Args:
            filename: Nombre del archivo
            
        Returns:
            True si es un archivo de texto, False en caso contrario
        """
        text_extensions = {
            '.txt', '.md', '.py', '.js', '.ts', '.html', '.css', '.scss', '.json',
            '.xml', '.yaml', '.yml', '.ini', '.cfg', '.conf', '.sh', '.bat',
            '.c', '.cpp', '.h', '.hpp', '.java', '.cs', '.go', '.rb', '.php'
        }
        
        return os.path.splitext(filename)[1].lower() in text_extensions
    
    def _count_code_lines(self, file_path: str) -> int:
        """
        Contar líneas de código en un archivo.
        
        Args:
            file_path: Ruta al archivo
            
        Returns:
            Número de líneas de código
        """
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                lines = f.readlines()
            
            # Filtrar líneas vacías y comentarios simples
            code_lines = [
                line for line in lines 
                if line.strip() and not line.strip().startswith(('#', '//', '/*', '*'))
            ]
            
            return len(code_lines)
        
        except Exception:
            return 0
    
    def _count_test_lines(self, file_path: str) -> int:
        """
        Contar líneas de código en un archivo de pruebas.
        
        Args:
            file_path: Ruta al archivo
            
        Returns:
            Número de líneas de pruebas
        """
        # Solo contar si es un archivo de prueba
        if 'test' not in os.path.basename(file_path).lower():
            return 0
        
        return self._count_code_lines(file_path)
    
    def _find_feature_specs(self) -> Dict[str, Any]:
        """
        Buscar especificaciones de características en el proyecto.
        
        Returns:
            Diccionario con características encontradas
        """
        specs = {}
        
        # Buscar en directorios típicos de documentación
        doc_dirs = ['docs', 'doc', 'specifications', 'specs', 'design']
        
        for doc_dir in doc_dirs:
            dir_path = os.path.join(self.project_path, doc_dir)
            
            if os.path.isdir(dir_path):
                for root, _, filenames in os.walk(dir_path):
                    for filename in filenames:
                        if filename.endswith(('.md', '.txt')):
                            file_path = os.path.join(root, filename)
                            
                            try:
                                with open(file_path, 'r', encoding='utf-8') as f:
                                    content = f.read()
                                
                                # Buscar títulos que sugieran una característica
                                feature_headers = re.findall(r'#+\s*(.+?)\n', content)
                                
                                for header in feature_headers:
                                    # Identificar palabras clave comunes en títulos de características
                                    keywords = ['feature', 'functionality', 'component', 'módulo', 
                                              'característica', 'funcionalidad']
                                    
                                    if any(keyword in header.lower() for keyword in keywords):
                                        # Extraer nombre de la característica
                                        name = re.sub(r'feature|functionality|component|:|-', '', 
                                                    header, flags=re.IGNORECASE).strip()
                                        
                                        # Añadir a especificaciones
                                        specs[name] = {
                                            "file": file_path,
                                            "description": header,
                                            "content": content
                                        }
                            
                            except Exception:
                                pass
        
        return specs
    
    def _estimate_feature_completion(self, feature_name: str, feature_files: List[str]) -> int:
        """
        Estimar porcentaje de completitud de una característica.
        
        Args:
            feature_name: Nombre de la característica
            feature_files: Archivos de la característica
            
        Returns:
            Porcentaje de completitud estimado
        """
        # Señales de completitud
        completion_signals = {
            "has_tests": any("test" in os.path.basename(f).lower() for f in feature_files),
            "has_docs": any(f.endswith(('.md', '.txt', '.html')) for f in feature_files),
            "has_implementation": any(self._is_code_file(f) for f in feature_files),
            "has_todo_marks": False,
            "has_issue_marks": False
        }
        
        # Buscar marcas TODO o FIXME en los archivos
        for file in feature_files:
            try:
                if self._is_text_file(file):
                    file_path = self._get_file_path(file)
                    with open(file_path, 'r', encoding='utf-8') as f:
                        content = f.read()
                    
                    if re.search(r'TODO|FIXME|XXX|PENDING', content, re.IGNORECASE):
                        completion_signals["has_todo_marks"] = True
                    
                    if re.search(r'ISSUE|BUG|PROBLEM', content, re.IGNORECASE):
                        completion_signals["has_issue_marks"] = True
            
            except Exception:
                pass
        
        # Calcular puntuación
        score = 0
        if completion_signals["has_implementation"]:
            score += 50  # Base por tener implementación
        
        if completion_signals["has_tests"]:
            score += 25  # Bonus por tests
        
        if completion_signals["has_docs"]:
            score += 15  # Bonus por documentación
        
        if completion_signals["has_todo_marks"]:
            score -= 15  # Penalización por TODOs
        
        if completion_signals["has_issue_marks"]:
            score -= 20  # Penalización por issues conocidas
        
        # Limitar entre 0 y 100
        return max(0, min(100, score))
    
    def _check_spec_implementation(self, spec: Dict[str, Any]) -> int:
        """
        Verificar implementación de una especificación.
        
        Args:
            spec: Diccionario con datos de la especificación
            
        Returns:
            Porcentaje de implementación estimado
        """
        try:
            content = spec.get("content", "")
            
            # Buscar menciones a requisitos o componentes
            requirements = re.findall(r'[-*]\s*(.+?)(?:\n|$)', content)
            
            if not requirements:
                return 50  # No se pueden identificar requisitos específicos
            
            # Verificar implementación de requisitos
            implemented = 0
            
            for req in requirements:
                # Extraer palabras clave del requisito
                keywords = [w for w in re.findall(r'\b\w{3,}\b', req.lower()) 
                           if w not in ['the', 'and', 'for', 'with', 'that', 'this']]
                
                if not keywords:
                    continue
                
                # Buscar estas palabras clave en archivos de código
                found = False
                for file in self.scanner.files:
                    if self._is_code_file(file):
                        try:
                            file_path = self._get_file_path(file)
                            with open(file_path, 'r', encoding='utf-8') as f:
                                file_content = f.read().lower()
                            
                            # Si se encuentran suficientes palabras clave, considerar implementado
                            if sum(1 for k in keywords if k in file_content) >= min(2, len(keywords)):
                                found = True
                                break
                        
                        except Exception:
                            pass
                
                if found:
                    implemented += 1
            
            # Calcular porcentaje
            return (implemented / len(requirements)) * 100
        
        except Exception as e:
            logger.warning(f"Error al verificar implementación de especificación: {str(e)}")
            return 0
    
    def _detect_architecture_pattern(self) -> str:
        """
        Detectar patrón arquitectónico del proyecto.
        
        Returns:
            Nombre del patrón detectado
        """
        directories = [d for d in os.listdir(self.project_path) 
                      if os.path.isdir(os.path.join(self.project_path, d))]
        
        # MVC: Models, Views, Controllers
        if all(d in directories for d in ['models', 'views', 'controllers']):
            return "MVC (Model-View-Controller)"
        
        # MVVM: Models, Views, ViewModels
        if all(d in directories for d in ['models', 'views', 'viewmodels']):
            return "MVVM (Model-View-ViewModel)"
        
        # Clean Architecture / Hexagonal
        if any(d in directories for d in ['domain', 'infrastructure', 'application']):
            return "Clean Architecture / Hexagonal"
        
        # Microservicios
        if any('service' in d.lower() for d in directories) and len(directories) > 5:
            return "Microservicios"
        
        # Monolítico por capas
        layers = ['data', 'services', 'ui', 'utils', 'helpers', 'api']
        if sum(1 for d in directories if any(layer in d.lower() for layer in layers)) >= 3:
            return "Monolítico por capas"
        
        return "Indeterminado"
    
    def _calculate_modularity_score(self) -> int:
        """
        Calcular puntuación de modularidad.
        
        Returns:
            Puntuación de modularidad (0-100)
        """
        try:
            # Obtener datos del grafo de dependencias
            graph_data = self.dependency_graph.build_dependency_graph(self.project_path)
            
            if not graph_data or not graph_data.get('nodes'):
                return 50  # Valor por defecto
            
            # Evaluar basado en características de la gráfica de dependencias
            num_nodes = len(graph_data['nodes'])
            
            if num_nodes == 0:
                return 50
            
            # Usar métricas del grafo
            metrics = graph_data.get('metrics', {})
            avg_deps = metrics.get('avg_out_degree', 0)
            
            # Identificar nodos centrales usando los datos existentes
            central_files = graph_data.get('central_files', [])
            central_ratio = min(len(central_files) / num_nodes, 1.0) if num_nodes > 0 else 0
            
            # Calcular puntuación
            # Menos dependencias por módulo = mejor modularity
            deps_score = max(0, 100 - (avg_deps * 10))
            
            # Menos nodos centrales = mejor modularidad
            central_score = max(0, 100 - (central_ratio * 100))
            
            # Combinar puntuaciones
            return int((deps_score * 0.6) + (central_score * 0.4))
            
        except Exception as e:
            logger.warning(f"Error calculando puntuación de modularidad: {str(e)}")
            return 50  # Valor por defecto en caso de error
    
    def _identify_risk_areas(self) -> List[Dict[str, Any]]:
        """
        Identificar áreas de riesgo en el código.
        
        Returns:
            Lista de áreas de riesgo
        """
        risk_areas = []
        
        # Archivos muy grandes
        for file in self.scanner.files:
            if self._is_code_file(file):
                try:
                    file_path = self._get_file_path(file)
                    with open(file_path, 'r', encoding='utf-8') as f:
                        content = f.read()
                    
                    lines = content.count('\n') + 1
                    
                    if lines > 500:
                        risk_areas.append({
                            "type": "large_file",
                            "file": file,
                            "lines": lines,
                            "risk_level": "medium" if lines < 1000 else "high"
                        })
                except Exception:
                    pass
        
        # Módulos con muchos dependientes
        try:
            graph_data = self.dependency_graph.build_dependency_graph(self.project_path)
            central_files = graph_data.get('central_files', [])
            
            for central_file in central_files:
                dependents = central_file.get('in_degree', 0)  # in_degree represents dependents
                if dependents > 10:
                    risk_areas.append({
                        "type": "high_coupling",
                        "file": central_file.get('file', ''),
                        "dependents": dependents,
                        "risk_level": "medium" if dependents < 20 else "high"
                    })
        except Exception as e:
            logger.warning(f"Error al analizar dependencias para áreas de riesgo: {str(e)}")
        
        # Ordenar por nivel de riesgo
        risk_areas.sort(key=lambda x: 0 if x["risk_level"] == "high" else 1)
        
        return risk_areas
    
    def _is_branch_old(self, date_str: str) -> bool:
        """
        Determinar si una branch no se ha actualizado recientemente.
        
        Args:
            date_str: Fecha de último commit en formato ISO
            
        Returns:
            True si la branch es antigua, False en caso contrario
        """
        if date_str == "N/A":
            return False
        
        try:
            # Convertir a fecha
            date_format = "%Y-%m-%d %H:%M:%S %z"
            if ' +' not in date_str and ' -' not in date_str:
                date_format = "%Y-%m-%d %H:%M:%S"
            
            commit_date = datetime.strptime(date_str, date_format)
            
            # Calcular días desde el commit
            delta = datetime.now() - commit_date.replace(tzinfo=None)
            days = delta.days
            
            # Considerar antigua si tiene más de 30 días
            return days > 30
        
        except Exception:
            return False
    
    def _get_file_path(self, file) -> str:
        """
        Obtener la ruta de un archivo, ya sea que se pase como string o como diccionario.
        
        Args:
            file: Ruta al archivo (str) o diccionario de información de archivo
            
        Returns:
            Ruta del archivo como string
        """
        if isinstance(file, dict) and 'path' in file:
            return file['path']
        elif isinstance(file, str):
            return file
        else:
            return str(file)  # Intentar convertir a string como último recurso
    
    def _create_basic_functional_groups(self) -> Dict[str, Any]:
        """
        Crear grupos funcionales básicos basados en funcionalidades detectadas.
        
        Returns:
            Dict con grupos funcionales básicos
        """
        features = {}
        
        try:
            # Usar detector de funcionalidades para crear grupos básicos
            from src.analyzers.functionality_detector import get_functionality_detector
            
            detector = get_functionality_detector()
            functionality_result = detector.detect_functionalities(self.project_path)
            
            detected_functionalities = functionality_result.get('detected', {})
            
            for func_name, func_data in detected_functionalities.items():
                if func_data.get('present', False):
                    evidence = func_data.get('evidence', {})
                    files = evidence.get('files', [])
                    
                    # Clean up and format the functionality name
                    clean_name = func_name.capitalize().strip()
                    
                    # Add appropriate emoji based on functionality type
                    icon_map = {
                        'authentication': '🔐',
                        'database': '🗄️',
                        'api': '🔗',
                        'frontend': '🎨',
                        'testing': '🧪',
                        'configuration': '⚙️',
                        'documentation': '📚',
                        'security': '🛡️',
                        'logging': '📋',
                        'deployment': '🚀',
                        'monitoring': '📊'
                    }
                    
                    icon = icon_map.get(func_name.lower(), '🔧')
                    display_name = f"{icon} {clean_name}"
                    
                    features[display_name] = {
                        "type": "functionality",
                        "files": len(files),
                        "description": f"Funcionalidad {func_name}",
                        "importance": func_data.get('confidence', 0),
                        "size": len(files),
                        "completion_estimate": min(func_data.get('confidence', 0), 100)
                    }
                    
        except Exception as e:
            logger.error(f"Error al crear grupos funcionales básicos: {e}")
            # Fallback a estructura de directorios simple
            features = self._create_directory_based_groups()
            
        return features
    
    def _create_directory_based_groups(self) -> Dict[str, Any]:
        """
        Crear grupos basados en estructura de directorios como último recurso.
        
        Returns:
            Dict con grupos basados en directorios
        """
        features = {}
        
        # Mapeo de directorios a tipos funcionales
        directory_mapping = {
            'src': '📁 Código Fuente Principal',
            'tests': '🧪 Pruebas y Testing',
            'docs': '📖 Documentación',
            'examples': '💡 Ejemplos y Demos',
            'config': '⚙️ Configuración',
            'scripts': '🔧 Scripts y Herramientas',
            'assets': '📦 Recursos y Assets',
            'public': '🌐 Archivos Públicos',
            'static': '📄 Archivos Estáticos'
        }
        
        try:
            dirs = [d for d in os.listdir(self.project_path) 
                    if os.path.isdir(os.path.join(self.project_path, d)) 
                    and not d.startswith('.') and d not in ('node_modules', 'venv', 'env', '__pycache__')]
            
            for dir_name in dirs:
                # Usar nombre mapeado si existe, sino usar el nombre del directorio
                display_name = directory_mapping.get(dir_name, f"📁 {dir_name.capitalize()}")
                
                dir_path = os.path.join(self.project_path, dir_name)
                feature_files = self._get_files_in_dir(dir_path)
                
                features[display_name] = {
                    "type": "directory",
                    "files": len(feature_files),
                    "description": f"Archivos en directorio {dir_name}",
                    "importance": len(feature_files) * 10,  # Importancia basada en número de archivos
                    "size": len(feature_files),
                    "completion_estimate": self._estimate_group_completion(feature_files)
                }
                
        except Exception as e:
            logger.error(f"Error al crear grupos de directorios: {e}")
            
        return features
    
    def _estimate_group_completion(self, group_files: List[str]) -> int:
        """
        Estimar porcentaje de completitud de un grupo funcional.
        
        Args:
            group_files: Lista de archivos en el grupo
            
        Returns:
            Porcentaje de completitud estimado
        """
        if not group_files:
            return 0
            
        # Señales de completitud
        completion_signals = {
            "has_tests": False,
            "has_docs": False,
            "has_implementation": False,
            "has_todo_marks": False,
            "has_issue_marks": False
        }
        
        # Analizar archivos del grupo
        for file_info in group_files:
            try:
                if isinstance(file_info, dict):
                    file_path = file_info.get('path', '')
                else:
                    file_path = str(file_info)
                
                if not file_path:
                    continue
                    
                file_name = os.path.basename(file_path).lower()
                
                # Detectar tipos de archivos
                if "test" in file_name:
                    completion_signals["has_tests"] = True
                
                if file_path.endswith(('.md', '.txt', '.html', '.rst')):
                    completion_signals["has_docs"] = True
                
                if self._is_code_file(file_path):
                    completion_signals["has_implementation"] = True
                    
                    # Buscar marcas TODO o FIXME si es un archivo de texto
                    if self._is_text_file(file_name):
                        try:
                            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                                content = f.read()
                            
                            if re.search(r'TODO|FIXME|XXX|PENDING', content, re.IGNORECASE):
                                completion_signals["has_todo_marks"] = True
                            
                            if re.search(r'ISSUE|BUG|PROBLEM', content, re.IGNORECASE):
                                completion_signals["has_issue_marks"] = True
                        except Exception:
                            pass
                
            except Exception as e:
                logger.debug(f"Error al analizar archivo {file_info}: {e}")
                continue
        
        # Calcular puntuación
        score = 0
        if completion_signals["has_implementation"]:
            score += 50  # Base por tener implementación
        
        if completion_signals["has_tests"]:
            score += 25  # Bonus por tests
        
        if completion_signals["has_docs"]:
            score += 15  # Bonus por documentación
        
        if completion_signals["has_todo_marks"]:
            score -= 15  # Penalización por TODOs
        
        if completion_signals["has_issue_marks"]:
            score -= 20  # Penalización por issues conocidas
        
        # Limitar entre 0 y 100
        return max(0, min(100, score))

def get_project_progress_tracker(project_path: str, config: Optional[ConfigManager] = None) -> ProjectProgressTracker:
    """
    Obtener un rastreador de progreso del proyecto.
    
    Args:
        project_path: Ruta al directorio del proyecto
        config: Configuración opcional
    
    Returns:
        Instancia de ProjectProgressTracker
    """
    return ProjectProgressTracker(project_path, config)



#!/usr/bin/env python3
"""
Script para corregir el uso de get_config() en el módulo de telemetría.
"""

import re

def replace_config_get():
    with open('src/utils/telemetry.py', 'r') as file:
        content = file.read()
    
    # Reemplazar la importación
    content = content.replace(
        'from src.utils.config import get_config, save_config',
        'from src.utils.config import config_manager, save_config'
    )
    
    # Reemplazar todas las ocurrencias de get_config()
    content = content.replace('config = get_config()', 'config = config_manager.config')
    
    # Reemplazar save_config(config) con config_manager.save_config()
    content = content.replace('save_config(config)', 'config_manager.save_config()')
    
    # Guardar los cambios
    with open('src/utils/telemetry.py', 'w') as file:
        file.write(content)
    
    print("Archivo src/utils/telemetry.py actualizado correctamente.")

if __name__ == '__main__':
    replace_config_get()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Módulo de plantillas de entrevistas para ProjectPrompt.

Este paquete contiene las plantillas de preguntas utilizadas por el sistema
de entrevistas guiadas para obtener información detallada sobre funcionalidades.
"""



#!/usr/bin/env python3
"""
Módulo de validación de licencias para ProjectPrompt.
Este módulo proporciona la funcionalidad para validar claves de licencia,
verificar su autenticidad y determinar el tipo de suscripción asociado.
"""

import os
import re
import json
import time
import base64
import hashlib
import hmac
import requests
from typing import Dict, Optional, Any, NamedTuple
from datetime import datetime, timedelta
from dataclasses import dataclass

from src.utils import logger, config_manager

# URL del servicio de verificación (simulado para desarrollo)
LICENSE_VERIFICATION_URL = "https://api.projectprompt.dev/v1/license/verify"

# Clave secreta para verificación offline (solo para desarrollo)
# En producción, usar un servicio web seguro para validación
_VERIFICATION_SECRET = "projectprompt_dev_license_validation_2023"


@dataclass
class LicenseStatus:
    """Clase para representar el estado de una licencia."""
    valid: bool = False
    expired: bool = False
    subscription_type: str = "free"
    expiration_date: Optional[str] = None
    user_name: Optional[str] = None
    user_email: Optional[str] = None


class LicenseValidator:
    """Clase para validar licencias de ProjectPrompt."""
    
    def __init__(self):
        """Inicializa el validador de licencias."""
        self.cache = {}
        self._last_online_check = 0
        
        # Intervalo mínimo entre verificaciones online (24 horas)
        self._online_check_interval = 24 * 60 * 60
    
    def _should_check_online(self) -> bool:
        """
        Determina si se debe realizar una verificación online.
        
        Returns:
            True si debe realizarse una verificación online, False en caso contrario
        """
        current_time = time.time()
        return (current_time - self._last_online_check) > self._online_check_interval
    
    def _verify_online(self, license_key: str) -> Optional[Dict[str, Any]]:
        """
        Verifica una licencia mediante el servicio online.
        
        Args:
            license_key: Clave de licencia a verificar
            
        Returns:
            Diccionario con la respuesta del servidor o None si hay un error
        """
        try:
            response = requests.post(
                LICENSE_VERIFICATION_URL,
                json={"license_key": license_key},
                timeout=5  # Timeout de 5 segundos
            )
            
            if response.status_code == 200:
                self._last_online_check = time.time()
                return response.json()
                
            logger.warning(f"Error al verificar licencia online. Estado: {response.status_code}")
            return None
        except Exception as e:
            logger.warning(f"No se pudo contactar al servidor de licencias: {e}")
            
            # Mostrar información útil al usuario sobre cómo resolver el problema
            if "NameResolutionError" in str(e) or "Failed to resolve" in str(e):
                logger.info("💡 Esto es normal - ProjectPrompt funciona completamente sin conexión.")
                logger.info("   Las funciones premium solo requieren claves API, no verificación de licencia online.")
            elif "Max retries exceeded" in str(e):
                logger.info("💡 Sin conexión a internet - ProjectPrompt continuará funcionando normalmente.")
                
            return None
    
    def _verify_offline(self, license_key: str) -> LicenseStatus:
        """
        Verifica una licencia de forma local (offline).
        Esta verificación es menos segura pero permite usar el software sin conexión.
        
        Args:
            license_key: Clave de licencia a verificar
            
        Returns:
            Estado de la licencia
        """
        status = LicenseStatus()
        
        # Una licencia válida tiene el formato: PREFIX-TYPE-EXPIRATION-HASH
        # Ejemplo: PP-PRO-20241231-a1b2c3d4e5f6
        
        pattern = r'^PP-(BASIC|PRO|TEAM)-(\d{8})-([a-zA-Z0-9]{12,16})$'
        match = re.match(pattern, license_key)
        
        if not match:
            logger.debug("El formato de la licencia no es válido")
            return status
            
        subscription_type = match.group(1).lower()
        expiration_date_str = match.group(2)
        verification_hash = match.group(3)
        
        try:
            # Verificar la fecha de expiración
            year = int(expiration_date_str[0:4])
            month = int(expiration_date_str[4:6])
            day = int(expiration_date_str[6:8])
            expiration_date = datetime(year, month, day)
            
            # Verificar si ha expirado
            if expiration_date < datetime.now():
                status.expired = True
                logger.debug(f"La licencia ha expirado el {expiration_date_str}")
                return status
                
            # Verificar la integridad de la clave
            data_to_verify = f"PP-{subscription_type.upper()}-{expiration_date_str}"
            expected_hash = self._generate_license_hash(data_to_verify)
            
            if verification_hash != expected_hash[:len(verification_hash)]:
                logger.debug("El hash de verificación de la licencia no coincide")
                return status
                
            # Si llegamos aquí, la licencia es válida
            status.valid = True
            status.subscription_type = subscription_type
            status.expiration_date = f"{year}-{month:02d}-{day:02d}"
            
            return status
            
        except ValueError:
            logger.debug("Formato de fecha inválido en la licencia")
            return status
    
    def _generate_license_hash(self, data: str) -> str:
        """
        Genera un hash para verificar la integridad de la licencia.
        
        Args:
            data: Datos a hashear
            
        Returns:
            Hash hexadecimal truncado
        """
        key = _VERIFICATION_SECRET.encode('utf-8')
        data_bytes = data.encode('utf-8')
        hmac_sha256 = hmac.new(key, data_bytes, hashlib.sha256)
        digest = hmac_sha256.hexdigest()
        return digest[:15]  # Truncar a 15 caracteres
    
    def validate_license(self, license_key: str) -> LicenseStatus:
        """
        Valida una clave de licencia y devuelve su estado.
        Primero intenta validarla online si es posible, luego recurre a validación offline.
        
        Args:
            license_key: Clave de licencia a validar
            
        Returns:
            Estado de la licencia
        """
        # Manejo de claves de demostración para pruebas
        if license_key in ["DEMO-LICENSE-KEY", "DEMO-BASIC", "DEMO-PRO", "DEMO-TEAM"]:
            status = LicenseStatus()
            status.valid = True
            if license_key == "DEMO-BASIC":
                status.subscription_type = "basic"
            elif license_key == "DEMO-PRO":
                status.subscription_type = "pro"
            elif license_key == "DEMO-TEAM":
                status.subscription_type = "team"
            else:
                status.subscription_type = "basic"
                
            # Establecer fecha de expiración a 30 días desde hoy para demo
            future_date = datetime.now() + timedelta(days=30)
            status.expiration_date = future_date.strftime("%Y-%m-%d")
            status.user_name = "Demo User"
            status.user_email = "demo@example.com"
            
            # Guardar en caché
            self.cache[license_key] = (status, time.time())
            return status
        
        # Verificar si tenemos una respuesta en caché
        if license_key in self.cache:
            cached_status, timestamp = self.cache[license_key]
            
            # Usar la caché si tiene menos de un día
            if (time.time() - timestamp) < 86400:  # 24 horas
                return cached_status
        
        status = LicenseStatus()
        
        # Intentar verificar online si corresponde
        online_success = False
        if self._should_check_online():
            online_result = self._verify_online(license_key)
            
            if online_result:
                # Actualizar estado con la respuesta online
                online_success = True
                status.valid = online_result.get("valid", False)
                status.expired = online_result.get("expired", False)
                status.subscription_type = online_result.get("subscription_type", "free")
                status.expiration_date = online_result.get("expiration_date")
                status.user_name = online_result.get("user_name")
                status.user_email = online_result.get("user_email")
                
                # Actualizar caché
                self.cache[license_key] = (status, time.time())
                return status
        
        # Si no pudimos verificar online, usar verificación offline
        offline_status = self._verify_offline(license_key)
        
        # Actualizar caché
        self.cache[license_key] = (offline_status, time.time())
        
        return offline_status


# Función de conveniencia para obtener la instancia del validador
def get_license_validator() -> LicenseValidator:
    """
    Obtiene una instancia del validador de licencias.
    
    Returns:
        Instancia del validador de licencias
    """
    return LicenseValidator()



#!/usr/bin/env python3
"""
Script para analizar la estructura del proyecto y sugerir mejoras de organización.
Este script utiliza el analizador de proyectos para analizar el proyecto actual
y proporcionar sugerencias para mejorar su estructura y organización.
"""

import os
import sys
import json
from pathlib import Path
from collections import defaultdict
import re

# Asegurarse de que podemos importar desde el directorio actual
sys.path.insert(0, str(Path(__file__).parent))

# Importaciones directas para evitar dependencias circulares
try:
    # Intentar importar el analizador de proyectos
    from src.analyzers.project_scanner import ProjectScanner
    from src.analyzers.functionality_detector import FunctionalityDetector
    has_src_imports = True
except ImportError as e:
    print(f"Advertencia: No se pueden importar módulos desde 'src': {e}")
    print("Cambiando a modo independiente usando quick_analyze.py...")
    has_src_imports = False

def analyze_project_structure(project_path):
    """Analiza la estructura del proyecto utilizando el analizador básico"""
    result = {
        "structure": [],
        "file_types": defaultdict(int),
        "total_files": 0,
        "potential_issues": []
    }
    
    try:
        # Como quick_analyze no devuelve los datos que necesitamos,
        # continuamos directamente con el análisis manual
        result = manual_project_analysis(project_path)
    except Exception as e:
        print(f"Error durante el análisis manual: {e}")
    
    return result

def manual_project_analysis(project_path):
    """Realiza un análisis manual básico del proyecto"""
    data = {
        "structure": [],
        "file_types": defaultdict(int),
        "total_files": 0,
        "potential_issues": []
    }
    
    # Analizar estructura de directorios
    for root, dirs, files in os.walk(project_path):
        # Ignorar directorios ocultos
        dirs[:] = [d for d in dirs if not d.startswith('.')]
        
        rel_path = os.path.relpath(root, project_path)
        if rel_path == '.':
            rel_path = ''
        
        # Contar archivos por tipo
        for file in files:
            if file.startswith('.'):
                continue
                
            data["total_files"] += 1
            _, ext = os.path.splitext(file)
            if ext:
                data["file_types"][ext.lower()] += 1
            else:
                data["file_types"]["sin_extension"] += 1
        
        # Registrar estructura
        data["structure"].append({
            "path": rel_path,
            "num_files": len(files),
            "num_dirs": len(dirs)
        })
    
    # Detectar posibles problemas
    detect_potential_issues(data, project_path)
    
    return data

def detect_potential_issues(data, project_path):
    """Detecta posibles problemas en la estructura del proyecto"""
    issues = []
    
    # Comprobar duplicación de código o directorios
    dir_counts = defaultdict(int)
    for item in data["structure"]:
        dir_name = os.path.basename(item["path"])
        if dir_name:
            dir_counts[dir_name] += 1
    
    # Registrar directorios duplicados
    for dir_name, count in dir_counts.items():
        if count > 1 and dir_name not in ('__pycache__', 'tests'):
            issues.append(f"Directorio duplicado: '{dir_name}' aparece {count} veces")
    
    # Comprobar directorios vacíos
    for item in data["structure"]:
        if item["num_files"] == 0 and item["num_dirs"] == 0:
            issues.append(f"Directorio vacío: '{item['path']}'")
    
    # Registrar resultados
    data["potential_issues"] = issues

def print_debug(msg):
    """Función auxiliar para asegurar que los mensajes se imprimen correctamente"""
    print(msg, flush=True)

def main():
    print_debug("Analizando estructura del proyecto y generando sugerencias de mejora...")
    print_debug("---------------------------------------------------------")
    
    # Obtener la ruta del proyecto (directorio actual)
    project_path = os.getcwd()
    
    try:
        # Analizar el proyecto
        print_debug("\nEjecutando análisis de estructura...")
        project_data = analyze_project_structure(project_path)
        
        print_debug("\n*** ANÁLISIS DE ESTRUCTURA DEL PROYECTO ***")
        print_debug("\nEste análisis identifica posibles mejoras en la organización del código.")
        print("Basado en el análisis estático del proyecto.\n")
        
        # Mostrar estadísticas generales
        print(f"Total de archivos: {project_data['total_files']}")
        print("\nDistribución de tipos de archivos:")
        for ext, count in sorted(project_data["file_types"].items(), key=lambda x: x[1], reverse=True)[:10]:
            print(f"  {ext}: {count} archivos")
        
        # Mostrar potenciales problemas
        if project_data.get("potential_issues"):
            print("\nProblemas potenciales detectados:")
            for issue in project_data["potential_issues"]:
                print(f"  - {issue}")
        else:
            print("\nNo se detectaron problemas estructurales evidentes.")
        
        print("\n*** SUGERENCIAS DE MEJORA DE ESTRUCTURA ***")
        
        # Analizar patrones y generar sugerencias
        suggestions = generate_structure_suggestions(project_data, project_path)
        
        if suggestions:
            for suggestion in suggestions:
                print(f"• {suggestion}")
        else:
            print("No se pudieron generar sugerencias específicas.")
            
    except Exception as e:
        print(f"\nError al generar el análisis: {str(e)}")
        print("Asegúrate de que tienes todas las dependencias instaladas.")
        print("Consulta la documentación para más información sobre la configuración.")

def generate_structure_suggestions(project_data, project_path):
    """Genera sugerencias para mejorar la estructura del proyecto basado en el análisis"""
    suggestions = []
    
    # Añadir sugerencias fijas basadas en patrones comunes
    suggestions.append("Hay múltiples scripts Python (.py) en el directorio raíz. Considera organizarlos en módulos o paquetes.")
    suggestions.append("Los archivos de shell script (.sh) podrían agruparse en un directorio 'scripts' o 'tools'.")
    suggestions.append("Hay directorios duplicados como 'src', 'docs' y 'tests' - considera consolidarlos.")
    suggestions.append("Elimina los directorios vacíos detectados para simplificar la estructura.")
    suggestions.append("Mueve los archivos de prueba 'test_*.py' a un directorio de tests unificado.")
    suggestions.append("Asegúrate de que cada archivo .py tenga un propósito claro en su nombre.")

    try:
        # Verificar si hay scripts duplicados o con funcionalidad similar
        script_files = []
        test_files = []
        
        # Analizar archivos en la raíz
        root_files = [f for f in os.listdir(project_path) if os.path.isfile(os.path.join(project_path, f))]
        py_files = [f for f in root_files if f.endswith('.py')]
        sh_files = [f for f in root_files if f.endswith('.sh')]
        
        if len(py_files) > 10:
            suggestions.append("Hay muchos scripts Python en el directorio raíz. Considera agruparlos en subdirectorios por funcionalidad.")
        
        if len(sh_files) > 5:
            suggestions.append("Hay varios scripts shell en el directorio raíz. Considera moverlos a un directorio 'scripts' o 'tools'.")        # Verificar estructura de tests
        test_dirs = []
        for root, dirs, files in os.walk(project_path):
            for d in dirs:
                if d.lower() in ('tests', 'test'):
                    test_dirs.append(os.path.join(root, d))
        
        if len(test_dirs) > 1:
            suggestions.append("Existen múltiples directorios de tests. Considera unificarlos en una estructura de tests coherente.")
        
        # Verificar redundancia en proyectos de prueba
        test_project_dirs = [d for d in os.listdir(project_path) if os.path.isdir(os.path.join(project_path, d)) and d.startswith('test-')]
        if len(test_project_dirs) > 2:
            suggestions.append("Hay múltiples directorios de proyectos de prueba. Considera consolidarlos o documentar claramente el propósito de cada uno.")
        
        # Verificar estructura de documentación
        docs_dir = os.path.join(project_path, 'docs')
        if os.path.exists(docs_dir) and os.path.isdir(docs_dir):
            doc_files = [f for f in os.listdir(docs_dir) if f.endswith('.md')]
            if len(doc_files) > 10:
                suggestions.append("La documentación contiene muchos archivos Markdown en el directorio principal. Considera organizarlos en subdirectorios temáticos.")
        
        # Verificar fases del proyecto
        fases_dir = os.path.join(project_path, 'fases')
        if os.path.exists(fases_dir) and os.path.isdir(fases_dir):
            suggestions.append("El directorio 'fases' parece contener documentación sobre el progreso del proyecto. Considera mover esta documentación a 'docs/development' o similar.")
    except Exception as e:
        print(f"Aviso: Error durante el análisis avanzado: {e}")
        # Continuar con sugerencias básicas
    
    # Sugerencias generales
    suggestions.extend([
        "Considera implementar un sistema de gestión de errores más robusto para evitar problemas de importación circular.",
        "Asegúrate de que los archivos de configuración tengan un formato consistente y estén en una ubicación central.",
        "Si la herramienta es para uso público, considera separar el código principal en un paquete instalable y la CLI en otro paquete.",
        "Elimina archivos .pyc y directorios __pycache__ antes de publicar o compartir el código.",
        "Consolida la funcionalidad de análisis que está dispersa en múltiples scripts similares."
    ])
    
    return suggestions

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Módulo para visualizar el dashboard de progreso del proyecto.

Este módulo implementa una interfaz de usuario para mostrar un dashboard 
con el progreso del proyecto, métricas y recomendaciones.
"""

import os
import sys
import json
from datetime import datetime
from typing import Dict, List, Any, Optional, Tuple
import webbrowser
import tempfile
import argparse

# Ensure local imports take precedence over system-installed packages
current_dir = os.path.dirname(os.path.abspath(__file__))
project_root = os.path.dirname(os.path.dirname(current_dir))
if project_root not in sys.path:
    sys.path.insert(0, project_root)

from src.analyzers.project_progress_tracker import ProjectProgressTracker, get_project_progress_tracker
from src.utils.logger import get_logger
from src.utils.config import ConfigManager
from src.utils.subscription_manager import get_subscription_manager

# Configuración del logger
logger = get_logger()


class DashboardGenerator:
    """
    Generador de dashboard para visualizar progreso de proyecto.
    
    Esta clase genera una representación visual del progreso del proyecto
    utilizando HTML/CSS para mostrar métricas, gráficos y recomendaciones.
    """
    
    def __init__(self, project_path: str, config: Optional[ConfigManager] = None):
        """
        Inicializar el generador del dashboard.
        
        Args:
            project_path: Ruta al directorio del proyecto
            config: Configuración opcional
        """
        self.project_path = os.path.abspath(project_path)
        self.config = config or ConfigManager()
        self.subscription = get_subscription_manager()
        self.tracker = get_project_progress_tracker(project_path, config)
        
        # Verificar acceso premium
        self.premium_access = self.subscription.is_premium_feature_available('project_dashboard')
        
        # Recursos para el dashboard
        self._css_template = self._get_css_template()
        self._js_template = self._get_js_template()
    
    def generate_dashboard(self, output_path: Optional[str] = None, open_browser: bool = True) -> str:
        """
        Generar el dashboard y guardarlo como HTML.
        
        Args:
            output_path: Ruta donde guardar el HTML (opcional)
            open_browser: Si es True, abre el dashboard en el navegador
            
        Returns:
            Ruta al archivo HTML generado
        """
        # Si no tiene acceso premium, generar versión reducida
        if not self.premium_access:
            return self._generate_free_dashboard(output_path, open_browser)
        
        # Recopilar todos los datos necesarios
        project_data = {
            "overview": self.tracker.get_project_overview(),
            "progress": self.tracker.get_progress_metrics(),
            "branches": self.tracker.get_branch_status(),
            "features": self.tracker.get_feature_progress(),
            "recommendations": self.tracker.get_recommendations(),
            "premium": True,
            "generated_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }
        
        # Generar HTML
        html = self._generate_html(project_data)
        
        # Determinar ruta de salida
        if not output_path:
            project_name = os.path.basename(self.project_path).replace(" ", "_")
            # Crear el directorio de análisis si no existe
            analyses_dir = os.path.join(self.project_path, "project-output", "analyses")
            os.makedirs(analyses_dir, exist_ok=True)
            output_path = os.path.join(analyses_dir, f"project_prompt_dashboard_{project_name}.html")
        
        # Guardar el archivo
        try:
            with open(output_path, 'w', encoding='utf-8') as f:
                f.write(html)
            logger.info(f"Dashboard generado en: {output_path}")
            
            # Abrir en el navegador si se solicitó
            if open_browser:
                webbrowser.open(f"file://{os.path.abspath(output_path)}")
            
            return output_path
            
        except Exception as e:
            logger.error(f"Error al guardar el dashboard: {str(e)}")
            raise
    
    def _generate_free_dashboard(self, output_path: Optional[str] = None, open_browser: bool = True) -> str:
        """
        Generar versión limitada del dashboard para usuarios free.
        
        Args:
            output_path: Ruta donde guardar el HTML (opcional)
            open_browser: Si es True, abre el dashboard en el navegador
            
        Returns:
            Ruta al archivo HTML generado
        """
        # Datos limitados
        project_data = {
            "overview": self.tracker.get_project_overview(),
            "progress": {
                "code_quality": self.tracker.get_progress_metrics().get("code_quality", {}),
                "testing": self.tracker.get_progress_metrics().get("testing", {})
            },
            "premium": False,
            "generated_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }
        
        # Generar HTML para versión free
        html = self._generate_html(project_data, is_free=True)
        
        # Determinar ruta de salida
        if not output_path:
            project_name = os.path.basename(self.project_path).replace(" ", "_")
            # Crear el directorio de análisis si no existe
            analyses_dir = os.path.join(self.project_path, "project-output", "analyses")
            os.makedirs(analyses_dir, exist_ok=True)
            output_path = os.path.join(analyses_dir, f"project_prompt_dashboard_{project_name}_free.html")
        
        # Guardar el archivo
        try:
            with open(output_path, 'w', encoding='utf-8') as f:
                f.write(html)
            logger.info(f"Dashboard (versión free) generado en: {output_path}")
            
            # Abrir en el navegador si se solicitó
            if open_browser:
                webbrowser.open(f"file://{os.path.abspath(output_path)}")
            
            return output_path
            
        except Exception as e:
            logger.error(f"Error al guardar el dashboard: {str(e)}")
            raise
    
    def _generate_html(self, data: Dict[str, Any], is_free: bool = False) -> str:
        """
        Generar HTML para el dashboard.
        
        Args:
            data: Datos a mostrar en el dashboard
            is_free: Si es la versión free o premium
            
        Returns:
            Código HTML del dashboard
        """
        # Template básico
        html_template = f"""<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Project Prompt Dashboard - {data['overview']['name']}</title>
    <style>
        {self._css_template}
    </style>
</head>
<body>
    <header>
        <div class="header-content">
            <h1>Project Prompt Dashboard</h1>
            <div class="project-name">{data['overview']['name']}</div>
            <div class="version-tag">{'Premium' if data.get('premium', False) else 'Free Version'}</div>
        </div>
    </header>
    
    <main>
        {self._generate_overview_section(data['overview'])}
        {self._generate_metrics_section(data.get('progress', {}))}
        {self._generate_branches_section(data.get('branches', {})) if not is_free else ''}
        {self._generate_features_section(data.get('features', {})) if not is_free else ''}
        {self._generate_recommendations_section(data.get('recommendations', [])) if not is_free else self._generate_upgrade_section()}
    </main>
    
    <footer>
        <p>Generado por ProjectPrompt el {data.get('generated_at', datetime.now().strftime("%Y-%m-%d %H:%M:%S"))}</p>
    </footer>
    
    <script>
        {self._js_template}
    </script>
</body>
</html>"""
        
        return html_template
    
    def _generate_overview_section(self, overview: Dict[str, Any]) -> str:
        """
        Generar la sección de visión general del proyecto.
        
        Args:
            overview: Datos de visión general
            
        Returns:
            HTML para la sección de visión general
        """
        # Gráfico de distribución de archivos por extensión
        extensions_data = overview.get('files', {}).get('by_extension', {})
        extensions_html = ""
        for ext, count in extensions_data.items():
            extensions_html += f'<div class="ext-item"><span class="ext-name">{ext}</span><span class="ext-count">{count}</span></div>'
        
        # Calcular porcentajes para el gráfico de código vs. comentarios
        code_metrics = overview.get('code_metrics', {})
        total_lines = code_metrics.get('total_lines', 0)
        code_lines = code_metrics.get('code_lines', 0)
        comment_lines = code_metrics.get('comment_lines', 0)
        
        code_percent = (code_lines / total_lines * 100) if total_lines > 0 else 0
        comment_percent = (comment_lines / total_lines * 100) if total_lines > 0 else 0
        other_percent = 100 - code_percent - comment_percent
        
        return f"""
        <section class="card overview">
            <h2>Visión General</h2>
            <div class="overview-stats">
                <div class="stat-item">
                    <div class="stat-value">{overview.get('files', {}).get('total', 0)}</div>
                    <div class="stat-label">Archivos</div>
                </div>
                <div class="stat-item">
                    <div class="stat-value">{code_metrics.get('total_lines', 0):,}</div>
                    <div class="stat-label">Líneas totales</div>
                </div>
                <div class="stat-item">
                    <div class="stat-value">{overview.get('structure', {}).get('directories', 0)}</div>
                    <div class="stat-label">Directorios</div>
                </div>
                <div class="stat-item">
                    <div class="stat-value">{code_metrics.get('files', 0)}</div>
                    <div class="stat-label">Archivos de código</div>
                </div>
            </div>
            
            <div class="overview-details">
                <div class="detail-column">
                    <h3>Distribución de archivos por extensión</h3>
                    <div class="extensions-list">
                        {extensions_html}
                    </div>
                </div>
                <div class="detail-column">
                    <h3>Distribución de líneas</h3>
                    <div class="code-distribution">
                        <div class="progress-bar">
                            <div class="progress-segment code" style="width: {code_percent}%" title="Código: {code_lines:,} líneas ({code_percent:.1f}%)"></div>
                            <div class="progress-segment comments" style="width: {comment_percent}%" title="Comentarios: {comment_lines:,} líneas ({comment_percent:.1f}%)"></div>
                            <div class="progress-segment other" style="width: {other_percent}%" title="Otros: {total_lines - code_lines - comment_lines:,} líneas ({other_percent:.1f}%)"></div>
                        </div>
                        <div class="distribution-legend">
                            <div class="legend-item">
                                <span class="color-box code"></span>
                                <span>Código ({code_percent:.1f}%)</span>
                            </div>
                            <div class="legend-item">
                                <span class="color-box comments"></span>
                                <span>Comentarios ({comment_percent:.1f}%)</span>
                            </div>
                            <div class="legend-item">
                                <span class="color-box other"></span>
                                <span>Otros ({other_percent:.1f}%)</span>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
            
            <div class="last-updated">
                <p>Última actualización: {overview.get('last_updated', 'Desconocido')}</p>
            </div>
        </section>
        """
    
    def _generate_metrics_section(self, progress: Dict[str, Any]) -> str:
        """
        Generar la sección de métricas de progreso.
        
        Args:
            progress: Datos de métricas de progreso
            
        Returns:
            HTML para la sección de métricas
        """
        # Métricas de completitud
        completeness = progress.get('completeness', {})
        completeness_score = completeness.get('score', 0)
        completeness_color = self._get_score_color(completeness_score)
        
        # Métricas de calidad
        code_quality = progress.get('code_quality', {})
        doc_percentage = code_quality.get('documentation_percentage', 0)
        doc_color = self._get_score_color(doc_percentage)
        
        # Métricas de testing
        testing = progress.get('testing', {})
        test_coverage = testing.get('coverage', 0)
        test_color = self._get_score_color(test_coverage)
        test_ratio = testing.get('ratio', 0)
        
        # Lista de archivos complejos
        complex_files = code_quality.get('complex_files', [])
        complex_files_html = ""
        for i, file in enumerate(complex_files[:5]):  # Mostrar máximo 5
            complex_files_html += f"""
            <tr>
                <td>{os.path.basename(file.get('file', ''))}</td>
                <td>{file.get('lines', 0)}</td>
                <td>{file.get('functions', 0)}</td>
                <td>{file.get('nested_depth', 0)}</td>
            </tr>
            """
        
        # Métricas avanzadas (solo premium)
        advanced_html = ""
        if 'advanced' in progress:
            advanced = progress['advanced']
            modularity_score = advanced.get('modularity_score', 0)
            modularity_color = self._get_score_color(modularity_score)
            
            architecture = advanced.get('architecture_pattern', 'Indeterminado')
            
            # Módulos centrales
            central_modules = advanced.get('central_modules', [])
            modules_html = ""
            for module in central_modules[:3]:  # Mostrar máximo 3
                modules_html += f"""
                <div class="module-item">
                    <span class="module-name">{os.path.basename(module.get('file', ''))}</span>
                    <span class="module-deps">{module.get('dependents', 0)} dependientes</span>
                </div>
                """
            
            advanced_html = f"""
            <div class="metrics-row">
                <div class="metric-card">
                    <h3>Modularidad</h3>
                    <div class="circular-progress" style="--progress: {modularity_score}%; --color: {modularity_color};">
                        <div class="inner">
                            <span class="number">{modularity_score}%</span>
                        </div>
                    </div>
                    <p class="description">Medida de independencia entre componentes</p>
                </div>
                
                <div class="metric-card">
                    <h3>Patrón arquitectónico</h3>
                    <div class="architecture-pattern">
                        <span class="pattern-name">{architecture}</span>
                    </div>
                    <p class="description">Patrón detectado en la estructura del proyecto</p>
                </div>
                
                <div class="metric-card">
                    <h3>Módulos centrales</h3>
                    <div class="central-modules">
                        {modules_html}
                    </div>
                    <p class="description">Componentes con mayor número de dependientes</p>
                </div>
            </div>
            """
        
        return f"""
        <section class="card metrics">
            <h2>Métricas de Progreso</h2>
            
            <div class="metrics-row">
                <div class="metric-card">
                    <h3>Completitud</h3>
                    <div class="circular-progress" style="--progress: {completeness_score}%; --color: {completeness_color};">
                        <div class="inner">
                            <span class="number">{completeness_score}%</span>
                        </div>
                    </div>
                    <p class="description">Componentes implementados vs. planificados</p>
                </div>
                
                <div class="metric-card">
                    <h3>Documentación</h3>
                    <div class="circular-progress" style="--progress: {doc_percentage}%; --color: {doc_color};">
                        <div class="inner">
                            <span class="number">{doc_percentage:.1f}%</span>
                        </div>
                    </div>
                    <p class="description">Porcentaje de código documentado</p>
                </div>
                
                <div class="metric-card">
                    <h3>Cobertura de tests</h3>
                    <div class="circular-progress" style="--progress: {test_coverage}%; --color: {test_color};">
                        <div class="inner">
                            <span class="number">{test_coverage:.1f}%</span>
                        </div>
                    </div>
                    <p class="description">Funciones con tests / total funciones</p>
                </div>
            </div>
            
            {advanced_html}
            
            <div class="complex-files">
                <h3>Archivos con alta complejidad</h3>
                <table class="data-table">
                    <thead>
                        <tr>
                            <th>Archivo</th>
                            <th>Líneas</th>
                            <th>Funciones</th>
                            <th>Profundidad</th>
                        </tr>
                    </thead>
                    <tbody>
                        {complex_files_html}
                    </tbody>
                </table>
            </div>
        </section>
        """
    
    def _generate_branches_section(self, branches_data: Dict[str, Any]) -> str:
        """
        Generar la sección del estado de branches.
        
        Args:
            branches_data: Datos de branches
            
        Returns:
            HTML para la sección de branches
        """
        if not branches_data or branches_data.get("error"):
            return f"""
            <section class="card branches">
                <h2>Estado de Branches</h2>
                <div class="error-message">
                    <p>{branches_data.get("error", "No se encontró información de branches.")}</p>
                </div>
            </section>
            """
        
        # Categorías de branches
        categories = branches_data.get("categories", {})
        categories_html = ""
        
        # Procesar cada categoría
        for category, branches in categories.items():
            if not branches:
                continue
                
            branches_list = ""
            for branch in branches:
                is_current = branch.get("current", False)
                name = branch.get("name", "")
                date = branch.get("last_commit_date", "")
                msg = branch.get("last_commit_msg", "")
                
                # Verificar si es antigua
                is_old = False
                try:
                    if date != "N/A":
                        date_format = "%Y-%m-%d %H:%M:%S %z"
                        if ' +' not in date and ' -' not in date:
                            date_format = "%Y-%m-%d %H:%M:%S"
                        commit_date = datetime.strptime(date, date_format)
                        delta = datetime.now() - commit_date.replace(tzinfo=None)
                        is_old = delta.days > 30
                except:
                    pass
                
                # Clase para branches actuales y antiguas
                class_names = []
                if is_current:
                    class_names.append("current")
                if is_old:
                    class_names.append("old")
                
                class_attr = f' class="{" ".join(class_names)}"' if class_names else ''
                
                branches_list += f"""
                <li{class_attr}>
                    <div class="branch-name">{name}</div>
                    <div class="branch-details">
                        <span class="commit-date">{date}</span>
                        <span class="commit-msg">{msg}</span>
                    </div>
                </li>
                """
            
            # Solo mostrar categorías con branches
            if branches_list:
                category_display = category.capitalize()
                categories_html += f"""
                <div class="branch-category">
                    <h3>{category_display} ({len(branches)})</h3>
                    <ul class="branch-list">
                        {branches_list}
                    </ul>
                </div>
                """
        
        return f"""
        <section class="card branches">
            <h2>Estado de Branches</h2>
            <div class="branches-container">
                {categories_html}
            </div>
        </section>
        """
    
    def _generate_features_section(self, features_data: Dict[str, Any]) -> str:
        """
        Generar la sección de progreso por características.
        
        Args:
            features_data: Datos de características
            
        Returns:
            HTML para la sección de características
        """
        if not features_data or features_data.get("error"):
            return f"""
            <section class="card features">
                <h2>Progreso por Características</h2>
                <div class="error-message">
                    <p>{features_data.get('error', 'No se encontraron características identificables.')}</p>
                </div>
            </section>
            """

        # Lista de características
        features = features_data.get("features", {})
        features_html = ""

        # Handle both dict and list types for features
        if isinstance(features, dict):
            items = features.items()
        elif isinstance(features, list):
            # If it's a list, convert to (name, data) pairs
            items = []
            for i, data in enumerate(features):
                name = data.get('name', f'Feature {i+1}') if isinstance(data, dict) else f'Feature {i+1}'
                items.append((name, data))
        else:
            items = []

        for name, data in items:
            # Defensive: if data is not a dict, skip
            if not isinstance(data, dict):
                continue
            completion = data.get("completion_estimate", 0)
            color = self._get_score_color(completion)
            files = data.get("files", 0)
            has_tests = "✓" if data.get("has_tests", False) else "✗"
            features_html += f"""
            <tr>
                <td>{name}</td>
                <td>
                    <div class="progress-bar small">
                        <div class="progress-fill" style="width: {completion}%; background-color: {color};" title="{completion}%"></div>
                    </div>
                </td>
                <td>{completion}%</td>
                <td>{files}</td>
                <td>{has_tests}</td>
            </tr>
            """

        return f"""
        <section class="card features">
            <h2>Progreso por Características</h2>
            <table class="data-table">
                <thead>
                    <tr>
                        <th>Característica</th>
                        <th>Progreso</th>
                        <th>Completitud</th>
                        <th>Archivos</th>
                        <th>Tests</th>
                    </tr>
                </thead>
                <tbody>
                    {features_html}
                </tbody>
            </table>
        </section>
        """
    
    def _generate_recommendations_section(self, recommendations: List[Dict[str, Any]]) -> str:
        """
        Generar la sección de recomendaciones.
        
        Args:
            recommendations: Lista de recomendaciones
            
        Returns:
            HTML para la sección de recomendaciones
        """
        if not recommendations:
            return """
            <section class="card recommendations">
                <h2>Recomendaciones</h2>
                <p class="no-recommendations">No hay recomendaciones en este momento.</p>
            </section>
            """
        
        recommendations_html = ""
        for rec in recommendations:
            priority = rec.get("priority", "medium")
            priority_class = f"priority-{priority}"
            
            recommendations_html += f"""
            <div class="recommendation-item {priority_class}">
                <div class="rec-header">
                    <span class="rec-type">{rec.get("type", "").capitalize()}</span>
                    <span class="rec-priority">{priority.capitalize()}</span>
                </div>
                <div class="rec-message">{rec.get("message", "")}</div>
                <div class="rec-action">{rec.get("action", "")}</div>
            </div>
            """
        
        return f"""
        <section class="card recommendations">
            <h2>Recomendaciones</h2>
            <div class="recommendations-container">
                {recommendations_html}
            </div>
        </section>
        """
    
    def _generate_upgrade_section(self) -> str:
        """
        Generar sección para actualizar a premium.
        
        Returns:
            HTML para la sección de actualización
        """
        return """
        <section class="card upgrade">
            <h2>Actualiza a Premium</h2>
            <div class="upgrade-content">
                <div class="upgrade-message">
                    <h3>Desbloquea todas las funcionalidades:</h3>
                    <ul>
                        <li>Estado detallado de branches y control de versiones</li>
                        <li>Progreso por características y componentes</li>
                        <li>Recomendaciones proactivas personalizadas</li>
                        <li>Métricas avanzadas de arquitectura y modularidad</li>
                        <li>Identificación de riesgos y áreas de mejora</li>
                    </ul>
                </div>
                <div class="upgrade-action">
                    <button class="upgrade-button" onclick="alert('Para actualizar a premium, ejecuta: project-prompt subscription plans')">Ver Planes Premium</button>
                </div>
            </div>
        </section>
        """
    
    def _get_score_color(self, score: float) -> str:
        """
        Obtener color según puntuación.
        
        Args:
            score: Puntuación (0-100)
            
        Returns:
            Color en formato hexadecimal
        """
        if score < 30:
            return "#e74c3c"  # Rojo
        elif score < 60:
            return "#f1c40f"  # Amarillo
        else:
            return "#2ecc71"  # Verde
    
    def _get_css_template(self) -> str:
        """
        Obtener plantilla CSS para el dashboard.
        
        Returns:
            Código CSS
        """
        return """
        :root {
            --primary: #3498db;
            --secondary: #2c3e50;
            --background: #f5f5f5;
            --card-bg: #ffffff;
            --text: #333333;
            --text-light: #7f8c8d;
            --border: #e0e0e0;
            --success: #2ecc71;
            --warning: #f1c40f;
            --danger: #e74c3c;
            --radius: 8px;
            --shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
        }
        
        * {
            margin: 0;
            padding: 0;
            box-sizing: border-box;
        }
        
        body {
            font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif;
            background-color: var(--background);
            color: var(--text);
            line-height: 1.6;
        }
        
        header {
            background-color: var(--primary);
            color: white;
            padding: 20px;
            box-shadow: var(--shadow);
        }
        
        .header-content {
            max-width: 1200px;
            margin: 0 auto;
            display: flex;
            flex-direction: column;
            align-items: center;
        }
        
        h1 {
            font-size: 28px;
            margin-bottom: 5px;
        }
        
        .project-name {
            font-size: 18px;
            opacity: 0.9;
            margin-bottom: 5px;
        }
        
        .version-tag {
            background-color: rgba(255, 255, 255, 0.2);
            padding: 4px 12px;
            border-radius: 20px;
            font-size: 14px;
            margin-top: 5px;
        }
        
        main {
            max-width: 1200px;
            margin: 20px auto;
            padding: 0 20px;
            display: grid;
            gap: 20px;
        }
        
        .card {
            background-color: var(--card-bg);
            border-radius: var(--radius);
            box-shadow: var(--shadow);
            padding: 20px;
            margin-bottom: 20px;
        }
        
        h2 {
            color: var(--secondary);
            margin-bottom: 20px;
            padding-bottom: 10px;
            border-bottom: 1px solid var(--border);
        }
        
        h3 {
            color: var(--secondary);
            margin-bottom: 15px;
            font-size: 18px;
        }
        
        /* Overview section */
        .overview-stats {
            display: grid;
            grid-template-columns: repeat(auto-fill, minmax(200px, 1fr));
            gap: 20px;
            margin-bottom: 30px;
        }
        
        .stat-item {
            text-align: center;
            padding: 15px;
            border-radius: var(--radius);
            border: 1px solid var(--border);
        }
        
        .stat-value {
            font-size: 30px;
            font-weight: bold;
            color: var(--primary);
            margin-bottom: 5px;
        }
        
        .stat-label {
            font-size: 15px;
            color: var(--text-light);
        }
        
        .overview-details {
            display: grid;
            grid-template-columns: 1fr 1fr;
            gap: 30px;
            margin-bottom: 20px;
        }
        
        @media (max-width: 768px) {
            .overview-details {
                grid-template-columns: 1fr;
            }
        }
        
        .extensions-list {
            display: grid;
            grid-template-columns: repeat(auto-fill, minmax(120px, 1fr));
            gap: 10px;
        }
        
        .ext-item {
            display: flex;
            justify-content: space-between;
            padding: 8px 12px;
            background-color: #f8f9fa;
            border-radius: 4px;
            font-size: 14px;
        }
        
        .ext-name {
            color: var(--secondary);
        }
        
        .ext-count {
            color: var(--primary);
            font-weight: bold;
        }
        
        .code-distribution {
            margin-top: 15px;
        }
        
        .progress-bar {
            height: 24px;
            background-color: #f0f0f0;
            border-radius: 4px;
            overflow: hidden;
            margin-bottom: 15px;
            display: flex;
        }
        
        .progress-bar.small {
            height: 10px;
        }
        
        .progress-segment {
            height: 100%;
        }
        
        .progress-segment.code {
            background-color: #3498db;
        }
        
        .progress-segment.comments {
            background-color: #2ecc71;
        }
        
        .progress-segment.other {
            background-color: #e0e0e0;
        }
        
        .progress-fill {
            height: 100%;
        }
        
        .distribution-legend {
            display: flex;
            justify-content: space-between;
        }
        
        .legend-item {
            display: flex;
            align-items: center;
            font-size: 13px;
        }
        
        .color-box {
            width: 12px;
            height: 12px;
            border-radius: 2px;
            margin-right: 5px;
        }
        
        .color-box.code {
            background-color: #3498db;
        }
        
        .color-box.comments {
            background-color: #2ecc71;
        }
        
        .color-box.other {
            background-color: #e0e0e0;
        }
        
        .last-updated {
            text-align: right;
            font-size: 13px;
            color: var(--text-light);
            margin-top: 20px;
        }
        
        /* Metrics section */
        .metrics-row {
            display: grid;
            grid-template-columns: repeat(auto-fit, minmax(250px, 1fr));
            gap: 20px;
            margin-bottom: 30px;
        }
        
        .metric-card {
            text-align: center;
            padding: 20px;
            border-radius: var(--radius);
            border: 1px solid var(--border);
        }
        
        .circular-progress {
            width: 120px;
            height: 120px;
            border-radius: 50%;
            background: conic-gradient(
                var(--color) calc(var(--progress) * 1%),
                #e0e0e0 0
            );
            margin: 0 auto 15px;
            display: flex;
            align-items: center;
            justify-content: center;
        }
        
        .circular-progress .inner {
            width: 80%;
            height: 80%;
            background-color: white;
            border-radius: 50%;
            display: flex;
            align-items: center;
            justify-content: center;
        }
        
        .circular-progress .number {
            font-size: 22px;
            font-weight: bold;
            color: var(--secondary);
        }
        
        .description {
            font-size: 14px;
            color: var(--text-light);
            margin-top: 10px;
        }
        
        .architecture-pattern {
            height: 120px;
            display: flex;
            align-items: center;
            justify-content: center;
            margin-bottom: 15px;
        }
        
        .pattern-name {
            font-size: 20px;
            color: var(--secondary);
            font-weight: bold;
        }
        
        .central-modules {
            height: 120px;
            display: flex;
            flex-direction: column;
            justify-content: center;
            margin-bottom: 15px;
        }
        
        .module-item {
            display: flex;
            justify-content: space-between;
            margin-bottom: 8px;
            padding: 5px 10px;
            background-color: #f8f9fa;
            border-radius: 4px;
        }
        
        .module-name {
            font-weight: bold;
            color: var(--secondary);
        }
        
        .module-deps {
            color: var(--primary);
        }
        
        /* Data tables */
        .data-table {
            width: 100%;
            border-collapse: collapse;
            margin-top: 15px;
        }
        
        .data-table th, .data-table td {
            padding: 10px;
            text-align: left;
            border-bottom: 1px solid var(--border);
        }
        
        .data-table th {
            background-color: #f8f9fa;
            font-weight: 600;
        }
        
        .data-table tr:hover {
            background-color: #f8f9fa;
        }
        
        /* Branches section */
        .branches-container {
            display: grid;
            grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
            gap: 20px;
        }
        
        .branch-list {
            list-style-type: none;
            margin-bottom: 20px;
        }
        
        .branch-list li {
            padding: 12px;
            border: 1px solid var(--border);
            border-radius: var(--radius);
            margin-bottom: 10px;
            background-color: #f8f9fa;
        }
        
        .branch-list li.current {
            border-left: 5px solid var(--primary);
        }
        
        .branch-list li.old {
            border-left: 5px solid var(--warning);
        }
        
        .branch-name {
            font-weight: bold;
            color: var(--secondary);
            margin-bottom: 5px;
        }
        
        .branch-details {
            display: flex;
            justify-content: space-between;
            font-size: 13px;
            color: var(--text-light);
        }
        
        /* Recommendations section */
        .recommendation-item {
            padding: 15px;
            border-left: 5px solid var(--primary);
            background-color: #f8f9fa;
            border-radius: var(--radius);
            margin-bottom: 15px;
        }
        
        .recommendation-item.priority-high {
            border-left-color: var(--danger);
        }
        
        .recommendation-item.priority-medium {
            border-left-color: var(--warning);
        }
        
        .recommendation-item.priority-low {
            border-left-color: var(--success);
        }
        
        .rec-header {
            display: flex;
            justify-content: space-between;
            margin-bottom: 10px;
        }
        
        .rec-type {
            font-weight: bold;
            color: var(--secondary);
        }
        
        .rec-priority {
            padding: 3px 10px;
            border-radius: 20px;
            font-size: 12px;
            background-color: #e0e0e0;
        }
        
        .priority-high .rec-priority {
            background-color: #fadbd8;
            color: #c0392b;
        }
        
        .priority-medium .rec-priority {
            background-color: #fef9e7;
            color: #b7950b;
        }
        
        .priority-low .rec-priority {
            background-color: #d5f5e3;
            color: #27ae60;
        }
        
        .rec-message {
            margin-bottom: 10px;
            font-size: 15px;
        }
        
        .rec-action {
            font-size: 14px;
            color: var(--primary);
            font-style: italic;
        }
        
        /* Upgrade section */
        .upgrade-content {
            display: grid;
            grid-template-columns: 2fr 1fr;
            gap: 20px;
            align-items: center;
        }
        
        @media (max-width: 768px) {
            .upgrade-content {
                grid-template-columns: 1fr;
            }
        }
        
        .upgrade-message h3 {
            margin-bottom: 10px;
        }
        
        .upgrade-message ul {
            padding-left: 20px;
        }
        
        .upgrade-message li {
            margin-bottom: 8px;
        }
        
        .upgrade-action {
            display: flex;
            justify-content: center;
            align-items: center;
        }
        
        .upgrade-button {
            background-color: var(--primary);
            color: white;
            border: none;
            padding: 12px 20px;
            border-radius: var(--radius);
            font-size: 16px;
            cursor: pointer;
            transition: background-color 0.3s;
        }
        
        .upgrade-button:hover {
            background-color: #2980b9;
        }
        
        /* Utility classes */
        .error-message {
            padding: 15px;
            background-color: #fadbd8;
            border-radius: var(--radius);
            color: #c0392b;
        }
        
        .no-recommendations {
            color: var(--text-light);
            font-style: italic;
            text-align: center;
            padding: 20px;
        }
        
        /* Footer */
        footer {
            background-color: var(--secondary);
            color: white;
            text-align: center;
            padding: 15px;
            margin-top: 30px;
            font-size: 14px;
        }
        """
    
    def _get_js_template(self) -> str:
        """
        Obtener plantilla JavaScript para el dashboard.
        
        Returns:
            Código JavaScript
        """
        return """
        // Dashboard interactivity
        document.addEventListener('DOMContentLoaded', function() {
            // Highlight current branch
            const currentBranch = document.querySelector('.branch-list li.current');
            if (currentBranch) {
                currentBranch.scrollIntoView({ behavior: 'smooth', block: 'center' });
            }
            
            // Make tables sortable
            const tables = document.querySelectorAll('.data-table');
            
            tables.forEach(table => {
                const headers = table.querySelectorAll('th');
                
                headers.forEach((header, index) => {
                    header.addEventListener('click', () => {
                        sortTable(table, index);
                    });
                    
                    // Add sort indicator
                    header.style.cursor = 'pointer';
                    header.setAttribute('title', 'Click to sort');
                });
            });
        });
        
        // Function to sort tables
        function sortTable(table, columnIndex) {
            let switching = true;
            let shouldSwitch, rows, i;
            let switchCount = 0;
            let direction = "asc";
            
            while (switching) {
                switching = false;
                rows = table.rows;
                
                for (i = 1; i < (rows.length - 1); i++) {
                    shouldSwitch = false;
                    
                    const x = rows[i].getElementsByTagName("td")[columnIndex];
                    const y = rows[i + 1].getElementsByTagName("td")[columnIndex];
                    
                    const xContent = x.textContent.toLowerCase();
                    const yContent = y.textContent.toLowerCase();
                    
                    // Convert to numbers if possible
                    const xValue = isNaN(Number(xContent)) ? xContent : Number(xContent);
                    const yValue = isNaN(Number(yContent)) ? yContent : Number(yContent);
                    
                    if (direction === "asc") {
                        if (xValue > yValue) {
                            shouldSwitch = true;
                            break;
                        }
                    } else {
                        if (xValue < yValue) {
                            shouldSwitch = true;
                            break;
                        }
                    }
                }
                
                if (shouldSwitch) {
                    rows[i].parentNode.insertBefore(rows[i + 1], rows[i]);
                    switching = true;
                    switchCount++;
                } else {
                    if (switchCount === 0 && direction === "asc") {
                        direction = "desc";
                        switching = true;
                    }
                }
            }
            
            // Update sort indicators
            const headers = table.querySelectorAll('th');
            headers.forEach(h => h.classList.remove('sorted-asc', 'sorted-desc'));
            
            headers[columnIndex].classList.add(direction === 'asc' ? 'sorted-desc' : 'sorted-asc');
        }
        """


class DashboardCLI:
    """Interfaz de línea de comandos para el dashboard."""
    
    def __init__(self):
        from src.utils.config import config_manager
        from src.utils.subscription_manager import get_subscription_manager
        
        self.config = config_manager
        self.subscription = get_subscription_manager()
    
    def run(self, args=None):
        """
        Ejecutar el CLI del dashboard.
        
        Args:
            args: Lista de argumentos de línea de comandos
            
        Returns:
            int: Código de salida (0 = éxito, 1 = error)
        """
        # Configurar argumentos
        parser = argparse.ArgumentParser(description="Generar dashboard del proyecto")
        parser.add_argument("--project", "-p", default=".", help="Ruta del proyecto")
        parser.add_argument("--output", "-o", help="Ruta donde guardar el dashboard")
        parser.add_argument("--format", "-f", choices=["html", "markdown", "md"], 
                          default="markdown", help="Formato de salida")
        parser.add_argument("--no-browser", action="store_true", 
                          help="No abrir automáticamente en el navegador")
        parser.add_argument("--premium", action="store_true", 
                          help="Modo premium con características avanzadas")
        parser.add_argument("--detailed", action="store_true",
                          help="Incluir análisis detallado de dependencias")
        
        # Procesar argumentos
        parsed_args = parser.parse_args(args or [])
        
        # Configurar banderas
        parsed_args.browser = not parsed_args.no_browser
        
        # Determinar ruta del proyecto
        project_path = parsed_args.project or os.getcwd()
        
        # Normalizar formato
        format_type = "markdown" if parsed_args.format in ["markdown", "md"] else "html"
        
        try:
            # Verificar acceso premium
            has_premium = self.subscription.is_premium_feature_available('project_dashboard')
            is_premium_mode = parsed_args.premium and has_premium
            
            if parsed_args.premium and not has_premium:
                print("⚠️  Las características premium requieren una suscripción activa.")
                print("    Ejecuta 'project-prompt subscription plans' para más información.\n")
                return 1
            
            if not is_premium_mode:
                print("⚠️  Dashboard simplificado (versión gratuita)")
                print("    El dashboard premium incluye:")
                print("    • Análisis detallado de dependencias y arquitectura")
                print("    • Seguimiento completo de branches con filtros inteligentes")
                print("    • Grupos funcionales con archivos específicos")
                print("    • Recomendaciones proactivas personalizadas")
                print("    Ejecuta 'project-prompt premium dashboard' para acceso completo.\n")
            else:
                print("✨ Dashboard Premium con análisis avanzado")
            
            print(f"📊 Generando dashboard {format_type.upper()} para el proyecto en {project_path}")
            
            # Generar dashboard según el formato
            if format_type == "markdown":
                # Importar y usar el generador de markdown
                from src.ui.markdown_dashboard import MarkdownDashboardGenerator
                
                markdown_generator = MarkdownDashboardGenerator(project_path, self.config)
                output_file = markdown_generator.generate_markdown_dashboard(
                    output_path=parsed_args.output,
                    premium_mode=is_premium_mode,
                    detailed=parsed_args.detailed
                )
                
                print(f"✅ Dashboard markdown generado correctamente en: {output_file}")
                
            else:
                # Usar el generador HTML existente
                dashboard = DashboardGenerator(project_path, self.config)
                output_file = dashboard.generate_dashboard(
                    output_path=parsed_args.output,
                    open_browser=parsed_args.browser,
                    premium_mode=is_premium_mode,
                    detailed=parsed_args.detailed
                )
                
                print(f"✅ Dashboard HTML generado correctamente en: {output_file}")
                
                if parsed_args.browser:
                    print("📱 Abriendo dashboard en el navegador...")
            
        except Exception as e:
            print(f"❌ Error al generar dashboard: {str(e)}")
            return 1
        
        return 0


def main():
    """Punto de entrada cuando se ejecuta como script."""
    cli = DashboardCLI()
    sys.exit(cli.run())


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Plantillas de entrevistas específicas para integración de APIs.

Este módulo contiene preguntas especializadas para funcionalidades
relacionadas con integración de APIs externas o internas.
"""

# Definición de plantillas para entrevistas sobre APIs
INTERVIEW_TEMPLATES = {
    # Plantilla general para APIs
    'general': {
        # Preguntas iniciales para APIs
        'initial_questions': [
            {
                'id': 'api_purpose',
                'question': '¿Cuál es el propósito principal de la integración con API para {functionality}?',
                'type': 'text'
            },
            {
                'id': 'api_type',
                'question': '¿Qué tipo de API se utiliza o se planea utilizar en {functionality}?',
                'type': 'choice',
                'options': [
                    'REST',
                    'GraphQL',
                    'SOAP',
                    'gRPC',
                    'WebSockets',
                    'Webhook',
                    'Otro'
                ]
            },
            {
                'id': 'api_provider',
                'question': '¿Es una API interna o externa? Si es externa, ¿quién es el proveedor?',
                'type': 'text'
            },
            {
                'id': 'api_endpoints',
                'question': '¿Qué endpoints o operaciones principales se utilizan en {functionality}?',
                'type': 'text'
            }
        ],
        
        # Preguntas de seguimiento para APIs
        'follow_up_questions': [
            {
                'id': 'api_auth',
                'question': '¿Qué método de autenticación utiliza la API en {functionality}?',
                'type': 'choice',
                'options': [
                    'API Key',
                    'OAuth/OAuth2',
                    'JWT',
                    'Autenticación básica',
                    'Sin autenticación',
                    'Otro'
                ]
            },
            {
                'id': 'api_rate_limits',
                'question': '¿Hay límites de tasa (rate limits) en la API que afecten a {functionality}?',
                'type': 'boolean'
            },
            {
                'id': 'api_rate_limit_details',
                'question': '¿Cuáles son los detalles de los límites de tasa de la API?',
                'type': 'text',
                'condition': {
                    'type': 'answer_boolean',
                    'question_id': 'api_rate_limits',
                    'is_true': True
                }
            },
            {
                'id': 'api_data_format',
                'question': '¿Qué formato de datos utiliza la API en {functionality}?',
                'type': 'choice',
                'options': ['JSON', 'XML', 'Protobuf', 'YAML', 'Texto plano', 'Binario', 'Otro']
            },
            {
                'id': 'api_error_handling',
                'question': '¿Cómo se manejan los errores de la API en {functionality}?',
                'type': 'text'
            },
            {
                'id': 'api_async',
                'question': '¿La integración con la API es síncrona o asíncrona?',
                'type': 'choice',
                'options': ['Síncrona', 'Asíncrona', 'Ambas', 'No definido']
            }
        ],
        
        # Preguntas finales para APIs
        'final_questions': [
            {
                'id': 'api_tests',
                'question': '¿Cómo se realizan o planean realizar las pruebas de integración con la API en {functionality}?',
                'type': 'text'
            },
            {
                'id': 'api_fallback',
                'question': '¿Existe algún plan de contingencia o fallback si la API no está disponible en {functionality}?',
                'type': 'text'
            },
            {
                'id': 'api_monitoring',
                'question': '¿Cómo se monitorea el rendimiento y disponibilidad de la API en {functionality}?',
                'type': 'text'
            },
            {
                'id': 'api_future',
                'question': '¿Hay planes futuros de cambios o evolución en la integración con la API en {functionality}?',
                'type': 'text'
            }
        ]
    },
    
    # Plantilla específica para APIs REST
    'rest_api': {
        'initial_questions': [
            {
                'id': 'rest_http_methods',
                'question': '¿Qué métodos HTTP se utilizan en la API REST para {functionality}?',
                'type': 'choice',
                'options': ['GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'HEAD', 'OPTIONS', 'Varios de los anteriores']
            },
            {
                'id': 'rest_resource_structure',
                'question': '¿Cuál es la estructura de recursos principal de la API REST en {functionality}?',
                'type': 'text'
            }
        ],
        'follow_up_questions': [
            {
                'id': 'rest_versioning',
                'question': '¿Cómo se maneja el versionado de la API REST en {functionality}?',
                'type': 'text'
            },
            {
                'id': 'rest_pagination',
                'question': '¿Se utiliza paginación en la API REST? Si es así, ¿cómo funciona?',
                'type': 'text'
            }
        ]
    },
    
    # Plantilla específica para APIs GraphQL
    'graphql_api': {
        'initial_questions': [
            {
                'id': 'graphql_schema',
                'question': '¿Cuál es el esquema GraphQL principal utilizado en {functionality}?',
                'type': 'text'
            },
            {
                'id': 'graphql_operations',
                'question': '¿Qué tipos de operaciones GraphQL (queries, mutations, subscriptions) se utilizan en {functionality}?',
                'type': 'text'
            }
        ],
        'follow_up_questions': [
            {
                'id': 'graphql_batching',
                'question': '¿Se utiliza batching o alguna estrategia de optimización de consultas en GraphQL para {functionality}?',
                'type': 'text'
            },
            {
                'id': 'graphql_fragments',
                'question': '¿Se utilizan fragmentos GraphQL en {functionality}? ¿Con qué finalidad?',
                'type': 'text'
            }
        ]
    }
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Módulo de funcionalidades avanzadas para la API de Anthropic (Claude).

Este módulo extiende las capacidades básicas de integración con Anthropic,
proporcionando funcionalidades premium como generación de código específico,
detección de errores y sugerencias de refactorización.
"""

import os
import json
from typing import Dict, List, Optional, Any, Union, Tuple
import logging
import requests
import tiktoken

from src.integrations.anthropic import AnthropicAPI, get_anthropic_client, ANTHROPIC_API_BASE_URL, ANTHROPIC_API_VERSION
from src.utils.logger import get_logger
from src.utils.config import ConfigManager
from src.utils.subscription_manager import get_subscription_manager
from src.utils.prompt_optimizer import get_prompt_optimizer

# Configurar logger
logger = get_logger()


class AdvancedAnthropicClient:
    """Cliente avanzado para la API de Anthropic con capacidades premium."""

    def __init__(self, api_key: Optional[str] = None, config: Optional[ConfigManager] = None):
        """
        Inicializar cliente avanzado de Anthropic.
        
        Args:
            api_key: Clave API opcional
            config: Objeto de configuración opcional
        """
        self.config = config or ConfigManager()
        self.base_client = get_anthropic_client(config)
        self.subscription = get_subscription_manager()
        self.prompt_optimizer = get_prompt_optimizer(config)
        
        # Modelos preferidos para tareas premium (configurables)
        self.code_model = self.config.get("api.anthropic.premium_model", "claude-3-opus-20240229")
        self.analysis_model = self.config.get("api.anthropic.analysis_model", "claude-3-sonnet-20240229")
    
    @property
    def is_configured(self) -> bool:
        """Comprobar si la API está correctamente configurada."""
        return self.base_client.is_configured
    
    def verify_premium_access(self) -> bool:
        """
        Verificar si el usuario tiene acceso a características premium.
        
        Returns:
            True si tiene acceso, False en caso contrario
        """
        # Verificar que el cliente base está configurado
        if not self.base_client.is_configured:
            logger.warning("Cliente de Anthropic no configurado para funciones premium")
            return False
        
        # Verificar suscripción activa para funciones premium de IA
        if not self.subscription.is_premium_feature_available("ai_integrations"):
            logger.warning("Usuario sin acceso a funciones premium de IA")
            return False
        
        return True
    
    def generate_code(self, 
                     prompt: str, 
                     language: str, 
                     context: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Generar código específico utilizando Claude.
        
        Args:
            prompt: Descripción de lo que se necesita generar
            language: Lenguaje de programación objetivo
            context: Contexto adicional para la generación
            
        Returns:
            Diccionario con el código generado y metadatos
        """
        # Verificar acceso premium
        if not self.verify_premium_access():
            return {
                "success": False,
                "error": "Esta función requiere suscripción premium",
                "code": None
            }
        
        # Contexto por defecto
        context = context or {}
        context["language"] = language
        
        # Optimizar el prompt para generación de código
        optimized_prompt = self.prompt_optimizer.optimize(
            prompt, "anthropic", "code_generation", context
        )
        
        # Crear mensaje para enviar a la API
        messages = [{
            "role": "user",
            "content": f"Generate {language} code for the following task: {optimized_prompt}\n\nPlease provide only well-structured, production-ready {language} code with appropriate comments."
        }]
        
        try:
            # Realizar solicitud a la API
            response = self._make_api_request(
                messages=messages,
                model=self.code_model,
                max_tokens=4000
            )
            
            # Extraer código de la respuesta
            code = self._extract_code_from_response(response, language)
            
            return {
                "success": True,
                "code": code,
                "language": language,
                "model": self.code_model
            }
        
        except Exception as e:
            logger.error(f"Error al generar código: {str(e)}")
            return {
                "success": False,
                "error": f"Error al generar código: {str(e)}",
                "code": None
            }
    
    def detect_errors(self, 
                     code: str, 
                     language: str,
                     context: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Detectar errores en código existente.
        
        Args:
            code: Código a analizar
            language: Lenguaje del código
            context: Contexto adicional para el análisis
            
        Returns:
            Diccionario con errores detectados y sugerencias
        """
        # Verificar acceso premium
        if not self.verify_premium_access():
            return {
                "success": False,
                "error": "Esta función requiere suscripción premium",
                "issues": []
            }
        
        # Contexto por defecto
        context = context or {}
        context["language"] = language
        
        # Optimizar el prompt para detección de errores
        code_prompt = f"Analyze this {language} code for bugs, security issues, and inefficiencies:\n\n```{language}\n{code}\n```"
        optimized_prompt = self.prompt_optimizer.optimize(
            code_prompt, "anthropic", "error_detection", context
        )
        
        # Crear mensaje para enviar a la API
        messages = [{
            "role": "user", 
            "content": f"{optimized_prompt}\n\nPlease identify issues in this format for each problem:\n1. [Issue Type]: Brief description\n   - Location: Where in the code\n   - Severity: (High/Medium/Low)\n   - Fix: Suggested solution"
        }]
        
        try:
            # Realizar solicitud a la API
            response = self._make_api_request(
                messages=messages,
                model=self.analysis_model,
                max_tokens=2000
            )
            
            # Procesar respuesta para extraer los problemas detectados
            issues = self._extract_issues_from_response(response)
            
            return {
                "success": True,
                "issues": issues,
                "language": language,
                "model": self.analysis_model
            }
        
        except Exception as e:
            logger.error(f"Error al detectar errores: {str(e)}")
            return {
                "success": False,
                "error": f"Error al analizar código: {str(e)}",
                "issues": []
            }
    
    def suggest_refactoring(self, 
                          code: str, 
                          language: str,
                          context: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Sugerir mejoras de refactorización para un código.
        
        Args:
            code: Código a refactorizar
            language: Lenguaje del código
            context: Contexto adicional para la refactorización
            
        Returns:
            Diccionario con sugerencias de refactorización
        """
        # Verificar acceso premium
        if not self.verify_premium_access():
            return {
                "success": False,
                "error": "Esta función requiere suscripción premium",
                "refactored_code": None,
                "suggestions": []
            }
        
        # Contexto por defecto
        context = context or {}
        context["language"] = language
        
        # Optimizar el prompt para sugerencias de refactorización
        code_prompt = f"Refactor this {language} code to improve its quality while maintaining functionality:\n\n```{language}\n{code}\n```"
        optimized_prompt = self.prompt_optimizer.optimize(
            code_prompt, "anthropic", "refactoring", context
        )
        
        # Crear mensaje para enviar a la API
        messages = [{
            "role": "user",
            "content": f"{optimized_prompt}\n\nPlease provide:\n1. An explanation of your refactoring approach\n2. The refactored code\n3. Key improvements made"
        }]
        
        try:
            # Realizar solicitud a la API
            response = self._make_api_request(
                messages=messages,
                model=self.code_model,
                max_tokens=4000
            )
            
            # Procesar respuesta
            refactored_code = self._extract_code_from_response(response, language)
            suggestions = self._extract_refactoring_suggestions(response)
            
            return {
                "success": True,
                "refactored_code": refactored_code,
                "suggestions": suggestions,
                "language": language,
                "model": self.code_model
            }
        
        except Exception as e:
            logger.error(f"Error al sugerir refactorización: {str(e)}")
            return {
                "success": False,
                "error": f"Error al generar sugerencias de refactorización: {str(e)}",
                "refactored_code": None,
                "suggestions": []
            }
    
    def explain_code(self, 
                    code: str, 
                    language: str,
                    detail_level: str = "standard",
                    context: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Generar explicación detallada de un código.
        
        Args:
            code: Código a explicar
            language: Lenguaje del código
            detail_level: Nivel de detalle ('basic', 'standard', 'advanced')
            context: Contexto adicional
            
        Returns:
            Diccionario con la explicación
        """
        # Verificar acceso premium para nivel avanzado
        if detail_level == "advanced" and not self.verify_premium_access():
            detail_level = "standard"
            logger.info("Nivel de detalle avanzado requiere suscripción premium, usando nivel estándar")
        
        # Determinar el modelo basado en el nivel de detalle
        model = self.analysis_model
        if detail_level == "basic":
            model = self.base_client.model  # Modelo básico
        
        # Crear mensaje para enviar a la API
        messages = [{
            "role": "user",
            "content": f"Explain this {language} code:\n\n```{language}\n{code}\n```\n\nDetail level: {detail_level}"
        }]
        
        try:
            # Realizar solicitud a la API
            response = self._make_api_request(
                messages=messages,
                model=model,
                max_tokens=2000
            )
            
            return {
                "success": True,
                "explanation": response.get("content", ""),
                "language": language,
                "model": model,
                "detail_level": detail_level
            }
        
        except Exception as e:
            logger.error(f"Error al explicar código: {str(e)}")
            return {
                "success": False,
                "error": f"Error al explicar código: {str(e)}",
                "explanation": None
            }
    
    def _make_api_request(self, 
                        messages: List[Dict[str, str]], 
                        model: str, 
                        max_tokens: int) -> Dict[str, Any]:
        """
        Realizar una solicitud a la API de Anthropic.
        
        Args:
            messages: Lista de mensajes a enviar
            model: Modelo de Anthropic a utilizar
            max_tokens: Número máximo de tokens para la respuesta
            
        Returns:
            Respuesta de la API
        """
        import time
        
        # Logging detallado para debugging
        logger.info("====== ANTHROPIC API REQUEST DEBUG ======")
        logger.info(f"API Key configured: {'Yes' if self.base_client.api_key else 'No'}")
        logger.info(f"API Key (first 10 chars): {self.base_client.api_key[:10] if self.base_client.api_key else 'None'}...")
        logger.info(f"Model: {model}")
        logger.info(f"Max tokens: {max_tokens}")
        logger.info(f"Messages count: {len(messages)}")
        if messages:
            logger.info(f"First message preview: {messages[0]['content'][:200]}...")
        
        # Verificar que tengamos una clave API válida
        if not self.base_client.api_key:
            logger.error("No se ha configurado una clave API para Anthropic")
            raise ValueError("No se ha configurado una clave API para Anthropic")
        
        # Preparar headers
        headers = {
            "x-api-key": self.base_client.api_key,
            "anthropic-version": ANTHROPIC_API_VERSION,
            "content-type": "application/json"
        }
        
        # Preparar datos de la solicitud
        data = {
            "model": model,
            "max_tokens": max_tokens,
            "messages": messages
        }
        
        url = f"{ANTHROPIC_API_BASE_URL}/v1/messages"
        logger.info(f"Making HTTP POST request to: {url}")
        
        # Timing de la solicitud
        start_time = time.time()
        
        # Realizar la solicitud
        try:
            response = requests.post(url, headers=headers, json=data)
            request_duration = time.time() - start_time
            
            logger.info(f"Request completed in {request_duration:.2f} seconds")
            logger.info(f"Response status code: {response.status_code}")
            logger.info(f"Response headers: {dict(response.headers)}")
            
            # Log response size
            if hasattr(response, 'content'):
                logger.info(f"Response content size: {len(response.content)} bytes")
            
        except Exception as e:
            logger.error(f"HTTP request failed: {str(e)}")
            raise
        
        # Verificar respuesta
        if response.status_code != 200:
            error_text = response.text[:500] if response.text else "No error text"
            logger.error(f"API returned non-200 status: {response.status_code}")
            logger.error(f"Error response text: {error_text}")
            error_info = response.json() if response.text else {"error": f"Status code: {response.status_code}"}
            raise Exception(f"Error en API Anthropic: {error_info}")
        
        try:
            response_data = response.json()
            logger.info(f"Response JSON keys: {list(response_data.keys()) if response_data else 'None'}")
            
            # Log usage info if available
            if "usage" in response_data:
                usage = response_data["usage"]
                logger.info(f"Token usage - Input: {usage.get('input_tokens', 'N/A')}, Output: {usage.get('output_tokens', 'N/A')}")
            
        except Exception as e:
            logger.error(f"Failed to parse response JSON: {str(e)}")
            logger.error(f"Raw response text: {response.text[:500]}")
            raise
        
        # Extraer contenido de la respuesta
        if response_data and "content" in response_data:
            content = response_data["content"][0]["text"] if response_data["content"] else ""
            logger.info(f"Response content length: {len(content)} characters")
            logger.info(f"Response content preview: {content[:200]}...")
            logger.info("====== API REQUEST COMPLETED ======")
            return {"content": content, "raw_response": response_data}
        else:
            logger.error(f"Unexpected response format: {response_data}")
            raise Exception("Formato de respuesta inesperado de la API")
    
    def _extract_code_from_response(self, response: Dict[str, Any], language: str) -> str:
        """
        Extraer código de la respuesta de la API.
        
        Args:
            response: Respuesta de la API
            language: Lenguaje del código
            
        Returns:
            Código extraído
        """
        content = response.get("content", "")
        
        # Buscar bloques de código con el lenguaje específico
        import re
        code_blocks = re.findall(r'```(?:' + language + r'|)\n(.*?)\n```', content, re.DOTALL)
        
        if code_blocks:
            # Devolver el primer bloque de código encontrado
            return code_blocks[0]
        
        # Si no hay bloques de código marcados, intentar encontrar el código por otros medios
        # Por ahora, devolvemos el contenido completo si no se encontraron bloques específicos
        return content
    
    def _extract_issues_from_response(self, response: Dict[str, Any]) -> List[Dict[str, Any]]:
        """
        Extraer problemas detectados de la respuesta de la API.
        
        Args:
            response: Respuesta de la API
            
        Returns:
            Lista de problemas detectados
        """
        content = response.get("content", "")
        issues = []
        
        # Patrones para extraer información de problemas
        import re
        issue_pattern = r'(\d+)\.\s+\[([^\]]+)\]:\s+([^\n]+)'
        detail_pattern = r'-\s+([^:]+):\s+([^\n]+)'
        
        # Encontrar todos los problemas
        matches = re.finditer(issue_pattern, content)
        
        for match in matches:
            issue_num = match.group(1)
            issue_type = match.group(2)
            description = match.group(3).strip()
            
            # Buscar detalles relacionados con este problema
            issue_content = content[match.end():]
            end_pos = issue_content.find(f"{int(issue_num)+1}. ")
            if end_pos == -1:
                end_pos = len(issue_content)
            
            issue_details = issue_content[:end_pos]
            
            # Extraer detalles
            location = ""
            severity = ""
            fix = ""
            
            for detail_match in re.finditer(detail_pattern, issue_details):
                detail_type = detail_match.group(1).strip().lower()
                detail_value = detail_match.group(2).strip()
                
                if "location" in detail_type:
                    location = detail_value
                elif "severity" in detail_type:
                    severity = detail_value
                elif "fix" in detail_type:
                    fix = detail_value
            
            # Añadir problema a la lista
            issues.append({
                "type": issue_type,
                "description": description,
                "location": location,
                "severity": severity,
                "fix": fix
            })
        
        return issues
    
    def _extract_refactoring_suggestions(self, response: Dict[str, Any]) -> List[str]:
        """
        Extraer sugerencias de refactorización de la respuesta de la API.
        
        Args:
            response: Respuesta de la API
            
        Returns:
            Lista de sugerencias
        """
        content = response.get("content", "")
        suggestions = []
        
        # Buscar sección de mejoras o sugerencias
        import re
        
        # Patrones comunes para identificar sugerencias
        patterns = [
            r'Key improvements(?:[^\n]*)?:(.*?)(?=\n\n|\n#|\n```|$)',
            r'Improvements(?:[^\n]*)?:(.*?)(?=\n\n|\n#|\n```|$)',
            r'Changes(?:[^\n]*)?:(.*?)(?=\n\n|\n#|\n```|$)',
            r'(?:^|\n)(\d+\.\s+[^\n]+)'
        ]
        
        for pattern in patterns:
            matches = re.findall(pattern, content, re.DOTALL)
            if matches:
                for match in matches:
                    # Limpiar y dividir en líneas si es una lista
                    if isinstance(match, str) and match.strip():
                        lines = [line.strip() for line in match.strip().split('\n') if line.strip()]
                        for line in lines:
                            # Si es un elemento de lista numerado
                            if re.match(r'^\d+\.\s+', line):
                                suggestions.append(line)
                            # Si es un elemento de lista con viñetas
                            elif line.startswith('- ') or line.startswith('* '):
                                suggestions.append(line)
                            # Si es texto normal
                            elif line and not line.startswith('#'):
                                suggestions.append(line)
        
        # Si no se encontraron sugerencias con los patrones anteriores
        if not suggestions:
            # Intentar extraer párrafos que parecen sugerencias
            potential_suggestions = [p for p in content.split('\n\n') 
                                     if 'improve' in p.lower() or 'refactor' in p.lower() 
                                     or 'better' in p.lower() or 'optimiz' in p.lower()]
            if potential_suggestions:
                suggestions = potential_suggestions[:3]  # Limitar a 3 sugerencias
        
        return suggestions


def get_advanced_anthropic_client(config: Optional[ConfigManager] = None) -> AdvancedAnthropicClient:
    """
    Obtener una instancia del cliente avanzado de Anthropic.
    
    Args:
        config: Configuración opcional
    
    Returns:
        Instancia de AdvancedAnthropicClient
    """
    return AdvancedAnthropicClient(config=config)



#!/usr/bin/env python3
"""
CLI wrapper para project-prompt

Este script proporciona una interfaz de línea de comandos para ProjectPrompt
"""

import os
import sys
import argparse
import subprocess

def main():
    """Punto de entrada principal para la CLI"""
    parser = argparse.ArgumentParser(description='ProjectPrompt: Análisis de proyectos')
    subparsers = parser.add_subparsers(dest='command', help='Comandos disponibles')
    
    # Comando analyze
    analyze_parser = subparsers.add_parser('analyze', help='Analizar un proyecto')
    analyze_parser.add_argument('path', nargs='?', default='.', help='Ruta al proyecto (por defecto: directorio actual)')
    analyze_parser.add_argument('-o', '--output', help='Archivo de salida para el análisis (formato JSON)')
    
    # Comando init
    init_parser = subparsers.add_parser('init', help='Inicializar un nuevo proyecto')
    
    # Procesar argumentos
    args = parser.parse_args()
    
    # Obtener directorio del script actual
    script_dir = os.path.dirname(os.path.abspath(__file__))
    
    if not args.command:
        parser.print_help()
        return
    
    if args.command == 'analyze':
        analyzer_path = os.path.join(script_dir, 'project_analyzer.py')
        cmd = [sys.executable, analyzer_path, args.path]
        if args.output:
            cmd.append(args.output)
        subprocess.run(cmd)
    
    elif args.command == 'init':
        print("Inicializando nuevo proyecto...")
        print("Esta función está pendiente de implementación.")

if __name__ == "__main__":
    main()



"""
Paquete de analizadores para proyectos.

Este paquete contiene los módulos encargados de analizar la estructura de un proyecto,
detectar sus características principales y generar informes detallados.
"""

from src.analyzers.project_scanner import ProjectScanner, get_project_scanner
from src.analyzers.file_analyzer import FileAnalyzer, get_file_analyzer
from src.analyzers.functionality_detector import FunctionalityDetector, get_functionality_detector
from src.analyzers.connection_analyzer import ConnectionAnalyzer, get_connection_analyzer
from src.analyzers.dependency_graph import DependencyGraph, get_dependency_graph
from src.analyzers.testability_analyzer import TestabilityAnalyzer, get_testability_analyzer
from src.analyzers.completeness_verifier import CompletenessVerifier, get_completeness_verifier
from src.analyzers.code_quality_analyzer import CodeQualityAnalyzer, get_code_quality_analyzer
from src.analyzers.project_progress_tracker import ProjectProgressTracker, get_project_progress_tracker
from src.analyzers.project_structure_analyzer import ProjectStructureAnalyzer, get_project_structure_analyzer
from src.analyzers.ai_insights_analyzer_lightweight import AIInsightsAnalyzer

__all__ = [
    'ProjectScanner', 'get_project_scanner',
    'FileAnalyzer', 'get_file_analyzer',
    'FunctionalityDetector', 'get_functionality_detector',
    'ConnectionAnalyzer', 'get_connection_analyzer',
    'ProjectProgressTracker', 'get_project_progress_tracker',
    'DependencyGraph', 'get_dependency_graph',
    'TestabilityAnalyzer', 'get_testability_analyzer',
    'CompletenessVerifier', 'get_completeness_verifier',
    'CodeQualityAnalyzer', 'get_code_quality_analyzer',
    'ProjectStructureAnalyzer', 'get_project_structure_analyzer',
    'AIInsightsAnalyzer',
]



#!/usr/bin/env python3
"""
Plantilla por defecto para tests en ProjectPrompt.
"""
import pytest
from datetime import datetime

# Test generado automáticamente el {{date}}

def test_imports():
    """Verificar que los módulos necesarios pueden ser importados."""
    try:
        # Aquí van los imports relevantes
        assert True
    except ImportError as e:
        assert False, f"Error de importación: {str(e)}"



#!/usr/bin/env python3
"""
Módulo para gestión de configuración en ProjectPrompt.
Permite cargar y guardar configuraciones desde archivos YAML,
y gestionar credenciales de APIs de forma segura.
"""

import os
import json
from pathlib import Path
from typing import Any, Dict, Optional, Union
import yaml

# Try to import python-dotenv for .env file support
try:
    from dotenv import load_dotenv
    DOTENV_AVAILABLE = True
except ImportError:
    DOTENV_AVAILABLE = False

# Try to import keyring, fall back to mock implementation if not available
try:
    import keyring
except ImportError:
    try:
        from src.utils import mock_keyring as keyring
    except ImportError:
        # If both fail, create a simple in-memory mock
        class MockKeyring:
            _storage = {}
            
            @staticmethod
            def get_password(service_name, username):
                return MockKeyring._storage.get((service_name, username))
                
            @staticmethod
            def set_password(service_name, username, password):
                MockKeyring._storage[(service_name, username)] = password
                
            @staticmethod
            def delete_password(service_name, username):
                if (service_name, username) in MockKeyring._storage:
                    del MockKeyring._storage[(service_name, username)]
                    return True
                return False
                
        keyring = MockKeyring

import logging

# Configure un logger básico para el módulo config
logger = logging.getLogger(__name__)

# Constantes
SERVICE_NAME = "project-prompt"
DEFAULT_CONFIG_DIR = os.path.expanduser("~/.config/project-prompt")
DEFAULT_CONFIG_FILE = "config.yaml"
DEFAULT_CONFIG_PATH = os.path.join(DEFAULT_CONFIG_DIR, DEFAULT_CONFIG_FILE)

# Valores por defecto de configuración
DEFAULT_CONFIG = {
    "log_level": "info",
    "api": {
        "anthropic": {
            "enabled": False,
            "key_name": "anthropic_API",
        },
        "openai": {
            "enabled": False,
            "key_name": "OPENAI_API_KEY",
        }
    },
    "features": {
        "premium": False
    }
}


class ConfigManager:
    """
    Gestor de configuración para ProjectPrompt.
    Maneja la carga, guardado y acceso a la configuración.
    """
    def __init__(
        self, 
        config_path: Optional[str] = None, 
        create_if_not_exists: bool = True
    ):
        """
        Inicializa el gestor de configuración.
        
        Args:
            config_path: Ruta al archivo de configuración
            create_if_not_exists: Si debe crear el archivo si no existe
        """
        self.config_path = config_path or DEFAULT_CONFIG_PATH
        self.config = DEFAULT_CONFIG.copy()
        
        # Load .env file if available
        self._load_env_file()
        
        # Crear directorio de configuración si no existe
        config_dir = os.path.dirname(self.config_path)
        if not os.path.exists(config_dir):
            try:
                os.makedirs(config_dir, exist_ok=True)
                logger.debug(f"Directorio de configuración creado: {config_dir}")
            except OSError as e:
                logger.error(f"Error al crear directorio de configuración: {e}")

        # Cargar configuración desde archivo o crear uno nuevo
        if os.path.exists(self.config_path):
            self.load_config()
        elif create_if_not_exists:
            logger.info(f"Archivo de configuración no encontrado, creando uno nuevo en {self.config_path}")
            self.save_config()
    
    def _load_env_file(self) -> None:
        """
        Carga variables de entorno desde archivos .env si están disponibles.
        Busca archivos .env en varios ubicaciones comunes.
        """
        env_paths = [
            ".env",  # Directorio actual
            os.path.join(os.getcwd(), ".env"),  # Directorio de trabajo actual
            os.path.join(os.path.expanduser("~"), ".env"),  # Directorio home
        ]
        
        # También buscar en el directorio padre del proyecto si estamos en un subdirectorio
        current_dir = os.getcwd()
        while current_dir != os.path.dirname(current_dir):  # Hasta llegar a la raíz
            env_path = os.path.join(current_dir, ".env")
            if env_path not in env_paths:
                env_paths.append(env_path)
            current_dir = os.path.dirname(current_dir)
        
        for env_path in env_paths:
            if os.path.exists(env_path):
                try:
                    if DOTENV_AVAILABLE:
                        from dotenv import load_dotenv
                        load_dotenv(env_path)
                        logger.debug(f"Archivo .env cargado desde {env_path}")
                    else:
                        # Manual loading if dotenv is not available
                        self._load_env_file_manual(env_path)
                        logger.debug(f"Archivo .env cargado manualmente desde {env_path}")
                    break
                except Exception as e:
                    logger.warning(f"Error al cargar archivo .env desde {env_path}: {e}")

    def _load_env_file_manual(self, env_path: str) -> None:
        """
        Carga manualmente un archivo .env sin usar python-dotenv.
        
        Args:
            env_path: Ruta al archivo .env
        """
        try:
            with open(env_path, 'r') as f:
                for line in f:
                    line = line.strip()
                    if line and not line.startswith('#') and '=' in line:
                        key, value = line.split('=', 1)
                        key = key.strip()
                        value = value.strip().strip('"\'')
                        # Solo establecer si no existe ya en el entorno
                        if key not in os.environ:
                            os.environ[key] = value
        except Exception as e:
            logger.warning(f"Error al cargar archivo .env manualmente: {e}")

    def load_config(self) -> Dict:
        """
        Carga la configuración desde el archivo.
        
        Returns:
            Diccionario con la configuración
        """
        try:
            with open(self.config_path, 'r') as config_file:
                loaded_config = yaml.safe_load(config_file)
                
                # Asegurarse que las claves predeterminadas existan
                if loaded_config is None:
                    loaded_config = {}
                
                # Fusionar con la configuración predeterminada para asegurar que todas las claves existan
                self._merge_configs(self.config, loaded_config)
                logger.debug(f"Configuración cargada desde {self.config_path}")
        except Exception as e:
            logger.error(f"Error al cargar la configuración: {e}")
        
        return self.config
    
    def save_config(self) -> bool:
        """
        Guarda la configuración actual en el archivo.
        
        Returns:
            True si la operación fue exitosa, False en caso contrario
        """
        try:
            with open(self.config_path, 'w') as config_file:
                yaml.dump(self.config, config_file, default_flow_style=False)
                logger.debug(f"Configuración guardada en {self.config_path}")
            return True
        except Exception as e:
            logger.error(f"Error al guardar la configuración: {e}")
            return False
    
    def get(self, key: str, default: Any = None) -> Any:
        """
        Obtiene un valor de la configuración.
        
        Args:
            key: Clave a obtener, puede ser en formato 'section.key'
            default: Valor por defecto si la clave no existe
            
        Returns:
            Valor de la configuración o el valor predeterminado
        """
        if '.' in key:
            parts = key.split('.')
            current = self.config
            for part in parts:
                if part not in current:
                    return default
                current = current[part]
            return current
        return self.config.get(key, default)
    
    def set(self, key: str, value: Any) -> None:
        """
        Establece un valor en la configuración.
        
        Args:
            key: Clave a establecer, puede ser en formato 'section.key'
            value: Valor a establecer
        """
        if '.' in key:
            parts = key.split('.')
            current = self.config
            for part in parts[:-1]:
                if part not in current:
                    current[part] = {}
                current = current[part]
            current[parts[-1]] = value
        else:
            self.config[key] = value
    
    def set_api_key(self, service: str, api_key: str) -> bool:
        """
        Almacena de forma segura la clave de API usando keyring.
        Si keyring no está disponible, almacena en la configuración (menos seguro).
        
        Args:
            service: Nombre del servicio (ej. 'anthropic', 'openai')
            api_key: Clave de API a almacenar
            
        Returns:
            True si la operación fue exitosa, False en caso contrario
        """
        try:
            # Intentar usar keyring
            try:
                keyring.set_password(SERVICE_NAME, service, api_key)
                logger.debug(f"Clave API para {service} almacenada en keyring")
            except Exception as ke:
                logger.warning(f"No se pudo usar keyring: {ke}. Almacenando en configuración (menos seguro).")
                # Fallback: almacenar en la configuración (menos seguro)
                key_section = f"api_keys.{service}"
                self.set(key_section, api_key)
            
            # Habilitar el servicio en la configuración
            if service == 'anthropic':
                self.set('api.anthropic.enabled', True)
            elif service == 'openai':
                self.set('api.openai.enabled', True)
            
            self.save_config()
            logger.info(f"Clave de API para {service} almacenada correctamente")
            return True
        except Exception as e:
            logger.error(f"Error al almacenar la clave de API para {service}: {e}")
            return False
    
    def get_api_key(self, service: str) -> Optional[str]:
        """
        Recupera la clave de API almacenada.
        Primero intenta desde keyring, luego desde la configuración, 
        y finalmente desde variables de entorno.
        
        Args:
            service: Nombre del servicio (ej. 'anthropic', 'openai')
            
        Returns:
            La clave de API o None si no existe
        """
        api_key = None
        
        # Intentar obtener desde keyring
        try:
            api_key = keyring.get_password(SERVICE_NAME, service)
            if api_key:
                logger.debug(f"Clave API para {service} recuperada desde keyring")
                return api_key
        except Exception as e:
            logger.debug(f"No se pudo usar keyring: {e}")
        
        # Si no está en keyring, intentar desde la configuración
        key_section = f"api_keys.{service}"
        api_key = self.get(key_section)
        if api_key:
            logger.debug(f"Clave API para {service} recuperada desde configuración")
            return api_key
        
        # Si no está en la configuración, intentar desde variables de entorno
        # Obtener el nombre de variable de entorno desde la configuración
        key_name = self.get(f"api.{service}.key_name")
        if key_name:
            api_key = os.getenv(key_name)
            if api_key:
                logger.debug(f"Clave API para {service} recuperada desde variable de entorno {key_name}")
                return api_key
        
        # También intentar algunas variables de entorno comunes
        common_env_names = {
            'anthropic': ['anthropic_API', 'ANTHROPIC_API_KEY'],
            'openai': ['OPENAI_API_KEY', 'openai_API']
        }
        
        if service in common_env_names:
            for env_name in common_env_names[service]:
                api_key = os.getenv(env_name)
                if api_key:
                    logger.debug(f"Clave API para {service} recuperada desde variable de entorno {env_name}")
                    return api_key
            
        logger.debug(f"No se encontró clave API para {service}")
        return api_key
    
    def delete_api_key(self, service: str) -> bool:
        """
        Elimina la clave de API almacenada tanto de keyring como de la configuración.
        
        Args:
            service: Nombre del servicio (ej. 'anthropic', 'openai')
            
        Returns:
            True si la operación fue exitosa, False en caso contrario
        """
        success = False
        
        # Intentar eliminar de keyring
        try:
            keyring.delete_password(SERVICE_NAME, service)
            success = True
            logger.debug(f"Clave API para {service} eliminada de keyring")
        except Exception as e:
            logger.debug(f"No se pudo eliminar clave de keyring: {e}")
        
        # También eliminar de la configuración por si acaso
        key_section = f"api_keys.{service}"
        if self.get(key_section):
            self.config.pop(key_section, None)
            success = True
            logger.debug(f"Clave API para {service} eliminada de configuración")
        
        # Guardar cambios en la configuración
        self.save_config()
        return success
    
    def is_premium(self) -> bool:
        """
        Verifica si la cuenta tiene acceso a funciones premium.
        
        Returns:
            True si tiene acceso a premium, False en caso contrario
        """
        return self.get("features.premium", False)
    
    def set_premium(self, enabled: bool) -> None:
        """
        Habilita o deshabilita el acceso a funciones premium.
        
        Args:
            enabled: True para habilitar, False para deshabilitar
        """
        self.set("features.premium", enabled)
        self.save_config()
        logger.info(f"Acceso a funciones premium {'habilitado' if enabled else 'deshabilitado'}")
    
    def _merge_configs(self, default: Dict, loaded: Dict) -> None:
        """
        Fusiona la configuración cargada con la configuración predeterminada.
        Args:
            default: Configuración predeterminada
            loaded: Configuración cargada
        """
        default_keys = list(default.keys())
        for key in default_keys:
            value = default[key]
            if key not in loaded:
                loaded[key] = value
            elif isinstance(value, dict) and isinstance(loaded[key], dict):
                self._merge_configs(value, loaded[key])
        self.config.update(loaded)


# Instancia global del gestor de configuración
config_manager = ConfigManager()

# Para uso más simple
get_config = config_manager.get
set_config = config_manager.set
save_config = config_manager.save_config
get_api_key = config_manager.get_api_key
set_api_key = config_manager.set_api_key
delete_api_key = config_manager.delete_api_key
is_premium = config_manager.is_premium
set_premium = config_manager.set_premium

# Alias for backward compatibility
Config = ConfigManager
